"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Decl = __importStar(require("../parser/declare"));
const Expr = __importStar(require("../parser/expr"));
const Stmt = __importStar(require("../parser/stmt"));
const treeTraverse_1 = require("../parser/treeTraverse");
const SuffixTerm = __importStar(require("../parser/suffixTerm"));
/**
 * Class to help identify parameters and return statements in
 * a kerboscript function
 */
class FunctionScan extends treeTraverse_1.TreeTraverse {
    /**
     * Function scan constructor
     */
    constructor() {
        super();
        this.result = {
            requiredParameters: 0,
            optionalParameters: 0,
            return: false,
        };
    }
    /**
     * Scan the function node for parameters and return statements
     * @param node function body
     */
    scan(node) {
        this.result = {
            requiredParameters: 0,
            optionalParameters: 0,
            return: false,
        };
        if (node instanceof Expr.Expr) {
            this.exprAction(node);
        }
        else if (node instanceof Stmt.Stmt) {
            this.stmtAction(node);
        }
        else if (node instanceof SuffixTerm.SuffixTerm) {
            this.suffixTermAction(node);
        }
        return { ...this.result };
    }
    /**
     * Action to apply at each node traversed
     * @param node node in question
     */
    nodeAction(node) {
        // traverse anonymous functions
        if (node instanceof Expr.Lambda) {
            return true;
        }
        // traverse statements
        if (node instanceof Stmt.Stmt) {
            // stop if we find another function
            if (node instanceof Decl.Func) {
                return false;
            }
            // indicate return statement exists
            if (node instanceof Stmt.Return) {
                this.result.return = true;
            }
            // Note this has no logic to detect parameters in loops
            // increment parameter count
            if (node instanceof Decl.Param) {
                this.result.requiredParameters += node.requiredParameters.length;
                this.result.optionalParameters += node.optionalParameters.length;
            }
            return true;
        }
        return false;
    }
}
exports.FunctionScan = FunctionScan;
//# sourceMappingURL=functionScan.js.map