import { IExprVisitor, IExpr, ISuffixTerm, ISuffixTermVisitor } from '../parser/types';
import * as Expr from '../parser/expr';
import * as SuffixTerm from '../parser/suffixTerm';
import { Token } from '../entities/token';
/**
 * Identify all local identifiers in a provided expression
 */
export declare class LocalResolver implements IExprVisitor<Token[]>, ISuffixTermVisitor<Token[]> {
    /**
     * Are we currently looking at a suffix trailer
     */
    private isTrailer;
    /**
     * local resolver constructor
     */
    constructor();
    /**
     * resolve local identifiers in expression
     * @param expr expression
     */
    resolveExpr(expr: IExpr): Token[];
    /**
     * resolve a suffix term
     * @param suffixTerm suffix term
     */
    resolveSuffixTerm(suffixTerm: ISuffixTerm): Token[];
    /**
     * Visit an invalid expression
     * @param _ invalid expression
     */
    visitExprInvalid(_: Expr.Invalid): Token[];
    /**
     * Visit a binary expression
     * @param expr binary expression
     */
    visitBinary(expr: Expr.Binary): Token[];
    /**
     * Visit a unary expression
     * @param expr unary expression
     */
    visitUnary(expr: Expr.Unary): Token[];
    /**
     * Vist a factor
     * @param expr factor expression
     */
    visitFactor(expr: Expr.Factor): Token[];
    /**
     * Vist a suffix expression
     * @param expr suffix expression
     */
    visitSuffix(expr: Expr.Suffix): Token[];
    /**
     * Visit a lambda
     * @param _ lambda expression
     */
    visitLambda(_: Expr.Lambda): Token[];
    /**
     * Vist an invalid suffix term
     * @param _ invalid suffix term
     */
    visitSuffixTermInvalid(_: SuffixTerm.Invalid): Token[];
    /**
     * visit a suffix term trailer
     * @param suffixTerm suffix term trailer
     */
    visitSuffixTrailer(suffixTerm: SuffixTerm.SuffixTrailer): Token[];
    /**
     * Visit a suffix term
     * @param suffixTerm suffix term
     */
    visitSuffixTerm(suffixTerm: SuffixTerm.SuffixTerm): Token[];
    /**
     * Visit a call suffix term trailer
     * @param suffixTerm call trailer
     */
    visitCall(suffixTerm: SuffixTerm.Call): Token[];
    /**
     * Visit an array index
     * @param _ array index trailer
     */
    visitArrayIndex(_: SuffixTerm.ArrayIndex): Token[];
    /**
     * Visit an array bracket
     * @param suffixTerm array bracket trailer
     */
    visitArrayBracket(suffixTerm: SuffixTerm.ArrayBracket): Token[];
    /**
     * Visit a delegate
     * @param _ delgate trailer
     */
    visitDelegate(_: SuffixTerm.Delegate): Token[];
    /**
     * Visit a literal
     * @param _ literal suffix term
     */
    visitLiteral(_: SuffixTerm.Literal): Token[];
    /**
     * Visit an identifier
     * @param suffixTerm suffix term identifier
     */
    visitIdentifier(suffixTerm: SuffixTerm.Identifier): Token[];
    /**
     * Visit a suffix term grouping
     * @param suffixTerm suffix term grouping
     */
    visitGrouping(suffixTerm: SuffixTerm.Grouping): Token[];
    /**
     * Execute a node as either a trailer or not. This function
     * returns to it's previous state post execution
     * @param isTrailer is the function a trailer
     * @param func function to execute
     */
    private executeAs;
}
