import { IExprVisitor, IExpr, ISuffixTerm, ISuffixTermVisitor } from '../parser/types';
import * as Expr from '../parser/expr';
import * as SuffixTerm from '../parser/suffixTerm';
import { LocalResolver } from './localResolver';
import { ISetResolverResult } from './types';
/**
 * Identify all local sybmbols used and all used symbols
 */
export declare class SetResolver implements IExprVisitor<ISetResolverResult>, ISuffixTermVisitor<ISetResolverResult> {
    readonly localResolver: LocalResolver;
    /**
     * Set resolver constructor
     */
    constructor(localResolver: LocalResolver);
    /**
     * Resolve expression
     * @param expr expression to resolve
     */
    resolveExpr(expr: IExpr): ISetResolverResult;
    /**
     * Resolve suffix term
     * @param suffixTerm suffix term to resolve
     */
    resolveSuffixTerm(suffixTerm: ISuffixTerm): ISetResolverResult;
    /**
     * Resolve invalid expression, i.e. do nothing
     * @param _ invalid expression
     */
    visitExprInvalid(_: Expr.Invalid): ISetResolverResult;
    /**
     * Resolve binary expresiosn. Binary expression cannot be set
     * @param _ binary expression
     */
    visitBinary(_: Expr.Binary): ISetResolverResult;
    /**
     * Resolve unary expression. Unary expression cannot be set
     * @param _ unary expression
     */
    visitUnary(_: Expr.Unary): ISetResolverResult;
    /**
     * Resolve factor expression. Factor cannot be set.
     * @param _ factor expression
     */
    visitFactor(_: Expr.Factor): ISetResolverResult;
    /**
     * Resolve suffix expression.
     * @param expr suffix expression
     */
    visitSuffix(expr: Expr.Suffix): ISetResolverResult;
    /**
     * Resolve lambda expression. Lambda cannot be set
     * @param _ lambda expression
     */
    visitLambda(_: Expr.Lambda): ISetResolverResult;
    /**
     * Resolve invalid suffix term. i.e. do nothing
     * @param _ invalid suffix term
     */
    visitSuffixTermInvalid(_: SuffixTerm.Invalid): ISetResolverResult;
    /**
     * Resolve suffix term trailer
     * @param expr suffix term trailer
     */
    visitSuffixTrailer(expr: SuffixTerm.SuffixTrailer): ISetResolverResult;
    /**
     * Resolve suffix term
     * @param expr suffix term
     */
    visitSuffixTerm(expr: SuffixTerm.SuffixTerm): ISetResolverResult;
    /**
     * Resolve suffix term call trailer
     * @param _ suffix term call trailer
     */
    visitCall(_: SuffixTerm.Call): ISetResolverResult;
    /**
     * Resolve array index trailer
     * @param _ suffix term array index trailer
     */
    visitArrayIndex(_: SuffixTerm.ArrayIndex): ISetResolverResult;
    /**
     * Resolve array bracket trailer
     * @param _ suffix term array bracket trailer
     */
    visitArrayBracket(_: SuffixTerm.ArrayBracket): ISetResolverResult;
    /**
     * Resolve delegate trailer
     * @param _ suffix term delegate
     */
    visitDelegate(_: SuffixTerm.Delegate): ISetResolverResult;
    /**
     * Resolve literal
     * @param _ literal
     */
    visitLiteral(_: SuffixTerm.Literal): ISetResolverResult;
    /**
     * Resolve identifier
     * @param expr identifier suffix term
     */
    visitIdentifier(expr: SuffixTerm.Identifier): ISetResolverResult;
    /**
     * Resolve grouping suffix term. Grouping cannot be set
     * @param _ grouping suffix term
     */
    visitGrouping(_: SuffixTerm.Grouping): ISetResolverResult;
}
