"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../parser/types");
const token_1 = require("../entities/token");
const tokentypes_1 = require("../entities/tokentypes");
const queue_1 = require("../typeChecker/types/collections/queue");
const structure_1 = require("../typeChecker/types/primitives/structure");
const list_1 = require("../typeChecker/types/collections/list");
const stack_1 = require("../typeChecker/types/collections/stack");
const uniqueset_1 = require("../typeChecker/types/collections/uniqueset");
const node_1 = require("../typeChecker/types/node");
const part_1 = require("../typeChecker/types/parts/part");
const constant_1 = require("../typeChecker/types/constant");
const path_1 = require("../typeChecker/types/io/path");
const volume_1 = require("../typeChecker/types/io/volume");
const vector_1 = require("../typeChecker/types/collections/vector");
const rgba_1 = require("../typeChecker/types/rgba");
const direction_1 = require("../typeChecker/types/direction");
const kacAlarmWrapper_1 = require("../typeChecker/types/kacAlarmWrapper");
const geoCoordinates_1 = require("../typeChecker/types/geoCoordinates");
const bodyatmosphere_1 = require("../typeChecker/types/bodyatmosphere");
const note_1 = require("../typeChecker/types/note");
const voice_1 = require("../typeChecker/types/voice");
const hsva_1 = require("../typeChecker/types/hsva");
const vectorRenderer_1 = require("../typeChecker/types/vectorRenderer");
const guiWidget_1 = require("../typeChecker/types/gui/guiWidget");
const orbitable_1 = require("../typeChecker/types/orbital/orbitable");
const timespan_1 = require("../typeChecker/types/timespan");
const highlight_1 = require("../typeChecker/types/highlight");
const orbitInfo_1 = require("../typeChecker/types/orbitInfo");
const career_1 = require("../typeChecker/types/career");
const waypoint_1 = require("../typeChecker/types/waypoint");
const resourceTransfer_1 = require("../typeChecker/types/resourceTransfer");
const constants_1 = require("../utilities/constants");
const lexicon_1 = require("../typeChecker/types/collections/lexicon");
const range_1 = require("../typeChecker/types/collections/range");
const symbolTableBuilder_1 = require("./symbolTableBuilder");
const volumneFile_1 = require("../typeChecker/types/io/volumneFile");
const pidLoop_1 = require("../typeChecker/types/pidLoop");
const volumeItem_1 = require("../typeChecker/types/io/volumeItem");
const volumeDirectory_1 = require("../typeChecker/types/io/volumeDirectory");
const typeCreators_1 = require("../typeChecker/typeCreators");
const delegate_1 = require("../typeChecker/types/primitives/delegate");
const kUniverse_1 = require("../typeChecker/types/kUniverse");
const homeConnection_1 = require("../typeChecker/types/communication/homeConnection");
const controlConnection_1 = require("../typeChecker/types/communication/controlConnection");
const vesselAlt_1 = require("../typeChecker/types/vessel/vesselAlt");
const vesselEta_1 = require("../typeChecker/types/vessel/vesselEta");
const stage_1 = require("../typeChecker/types/vessel/stage");
const steeringManager_1 = require("../typeChecker/types/steeringManager");
const terminalStruct_1 = require("../typeChecker/types/terminalStruct");
const void_1 = require("../typeChecker/types/primitives/void");
const userList_1 = require("../typeChecker/types/collections/userList");
const scalar_1 = require("../typeChecker/types/primitives/scalar");
const string_1 = require("../typeChecker/types/primitives/string");
const boolean_1 = require("../typeChecker/types/primitives/boolean");
const core_1 = require("../typeChecker/types/core");
const versionInfo_1 = require("../typeChecker/types/versionInfo");
const config_1 = require("../typeChecker/types/config");
const builtInDelegate_1 = require("../typeChecker/types/builtInDelegate");
const addonList_1 = require("../typeChecker/types/addonList");
const vesselSensors_1 = require("../typeChecker/types/vessel/vesselSensors");
const serializeableStructure_1 = require("../typeChecker/types/primitives/serializeableStructure");
const bodyTarget_1 = require("../typeChecker/types/orbital/bodyTarget");
const vesselTarget_1 = require("../typeChecker/types/orbital/vesselTarget");
const stringUtils_1 = require("../utilities/stringUtils");
const marker_1 = require("../entities/marker");
const functionTypes = [
    [['abs'], typeCreators_1.createFunctionType('abs', scalar_1.scalarType, scalar_1.scalarType)],
    [['add'], typeCreators_1.createFunctionType('add', void_1.voidType, node_1.nodeType)],
    [
        ['add', 'alarm'],
        typeCreators_1.createFunctionType('addalarm', kacAlarmWrapper_1.kacAlarmType, string_1.stringType, string_1.stringType, scalar_1.scalarType, string_1.stringType),
    ],
    [
        ['all', 'waypoints'],
        typeCreators_1.createFunctionType('allwaypoints', list_1.listType.toConcreteType(waypoint_1.waypointType)),
    ],
    [
        ['angle', 'axis'],
        typeCreators_1.createFunctionType('angleaxis', direction_1.directionType, vector_1.vectorType, scalar_1.scalarType),
    ],
    [
        ['angle', 'diff'],
        typeCreators_1.createFunctionType('anglediff', scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType),
    ],
    [['arc', 'cos'], typeCreators_1.createFunctionType('arccos', scalar_1.scalarType)],
    [['arc', 'sin'], typeCreators_1.createFunctionType('arcsin', scalar_1.scalarType)],
    [['arc', 'tan'], typeCreators_1.createFunctionType('arctan', scalar_1.scalarType)],
    [['arc', 'tan2'], typeCreators_1.createFunctionType('arctan2', scalar_1.scalarType)],
    // TODO need to figure out name collision for scope manager
    // createFunctionType('body', bodyTargetType, stringType)],
    [
        ['body', 'atmosphere'],
        typeCreators_1.createFunctionType('bodyatmosphere', bodyatmosphere_1.bodyAtmosphereType, string_1.stringType),
    ],
    [
        ['build', 'list'],
        typeCreators_1.createFunctionType('buildlist', /* TODO Union */ scalar_1.scalarType, string_1.stringType),
    ],
    [['career'], typeCreators_1.createFunctionType('career', career_1.careerType)],
    [['cd'], typeCreators_1.createFunctionType('cd', void_1.voidType, string_1.stringType)],
    [['ceiling'], typeCreators_1.createFunctionType('ceiling', scalar_1.scalarType, scalar_1.scalarType)],
    [['char'], typeCreators_1.createFunctionType('char', string_1.stringType, scalar_1.scalarType)],
    [['chdir'], typeCreators_1.createFunctionType('chdir', void_1.voidType, string_1.stringType)],
    [['clear', 'guis'], typeCreators_1.createFunctionType('clearguis', void_1.voidType)],
    [['clear', 'screen'], typeCreators_1.createFunctionType('clearscreen', void_1.voidType)],
    [['clear', 'vec', 'draws'], typeCreators_1.createFunctionType('clearvecdraws', void_1.voidType)],
    [['constant'], typeCreators_1.createFunctionType('constant', constant_1.constantType)],
    [
        ['copy', 'path'],
        typeCreators_1.createFunctionType('copypath', void_1.voidType, string_1.stringType, string_1.stringType),
    ],
    [['cos'], typeCreators_1.createFunctionType('cos', scalar_1.scalarType)],
    [['create'], typeCreators_1.createFunctionType('create', volumneFile_1.volumeFileType, string_1.stringType)],
    [
        ['create', 'dir'],
        typeCreators_1.createFunctionType('createdir', volumeDirectory_1.volumeDirectoryType, string_1.stringType),
    ],
    [['debug', 'dump'], typeCreators_1.createFunctionType('debugdump', void_1.voidType)],
    [
        ['debug', 'freeze', 'game'],
        typeCreators_1.createFunctionType('debugfreezegame', scalar_1.scalarType),
    ],
    [
        ['delete', 'alarm'],
        typeCreators_1.createFunctionType('deleteAlarm', boolean_1.booleanType, string_1.stringType),
    ],
    [
        ['delete', 'path'],
        typeCreators_1.createFunctionType('deletepath', void_1.voidType, string_1.stringType, string_1.stringType),
    ],
    [['edit'], typeCreators_1.createFunctionType('edit', void_1.voidType, path_1.pathType)],
    [['exists'], typeCreators_1.createFunctionType('exists', boolean_1.booleanType, string_1.stringType)],
    [['floor'], typeCreators_1.createFunctionType('floor', scalar_1.scalarType, scalar_1.scalarType)],
    [['get', 'voice'], typeCreators_1.createFunctionType('getvoice', voice_1.voiceType, scalar_1.integarType)],
    [['gui'], typeCreators_1.createFunctionType('gui', guiWidget_1.guiWidgetType, scalar_1.integarType, scalar_1.integarType)],
    [
        ['heading'],
        typeCreators_1.createFunctionType('heading', direction_1.directionType, scalar_1.scalarType, scalar_1.scalarType),
    ],
    [
        ['highlight'],
        typeCreators_1.createFunctionType('highlight', highlight_1.highlightType, structure_1.structureType, rgba_1.rgbaType),
    ],
    [
        ['hsv'],
        typeCreators_1.createFunctionType('hsv', hsva_1.hsvaType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType),
    ],
    [
        ['hsva'],
        typeCreators_1.createFunctionType('hsva', hsva_1.hsvaType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType),
    ],
    [
        ['hudtext'],
        typeCreators_1.createFunctionType('hudtext', void_1.voidType, string_1.stringType, scalar_1.integarType, scalar_1.integarType, rgba_1.rgbaType, boolean_1.booleanType),
    ],
    [
        ['latlng'],
        typeCreators_1.createFunctionType('latlng', geoCoordinates_1.geoCoordinatesType, scalar_1.scalarType, scalar_1.scalarType),
    ],
    [
        ['lex'],
        typeCreators_1.createVarFunctionType('lex', lexicon_1.lexiconType, typeCreators_1.createVarType(structure_1.structureType)),
    ],
    [
        ['lexicon'],
        typeCreators_1.createVarFunctionType('lexicon', lexicon_1.lexiconType, typeCreators_1.createVarType(structure_1.structureType)),
    ],
    [
        ['list'],
        typeCreators_1.createVarFunctionType('list', userList_1.userListType, typeCreators_1.createVarType(structure_1.structureType)),
    ],
    [
        ['list', 'alarms'],
        typeCreators_1.createFunctionType('listAlarms', list_1.listType.toConcreteType(kacAlarmWrapper_1.kacAlarmType), string_1.stringType),
    ],
    [['ln'], typeCreators_1.createFunctionType('ln', scalar_1.scalarType, scalar_1.scalarType)],
    [['log10'], typeCreators_1.createFunctionType('log10', scalar_1.scalarType, scalar_1.scalarType)],
    [
        ['log', 'file'],
        typeCreators_1.createFunctionType('logfile', void_1.voidType, string_1.stringType, string_1.stringType),
    ],
    [
        ['look', 'dir', 'up'],
        typeCreators_1.createFunctionType('lookdirup', direction_1.directionType, vector_1.vectorType, vector_1.vectorType),
    ],
    [
        ['make', 'builtin', 'delegate'],
        typeCreators_1.createFunctionType('makebuiltindelegate', builtInDelegate_1.builtInDelegateType, string_1.stringType),
    ],
    [['max'], typeCreators_1.createFunctionType('max', scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType)],
    [['min'], typeCreators_1.createFunctionType('min', scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType)],
    [['mod'], typeCreators_1.createFunctionType('mod', scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType)],
    [
        ['move', 'path'],
        typeCreators_1.createFunctionType('movepath', void_1.voidType, string_1.stringType, string_1.stringType),
    ],
    [
        ['node'],
        typeCreators_1.createFunctionType('node', node_1.nodeType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType),
    ],
    [
        ['note'],
        typeCreators_1.createFunctionType('note', note_1.noteType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType),
    ],
    [['open'], typeCreators_1.createFunctionType('open', volumeItem_1.volumeItemType, string_1.stringType)],
    [
        ['orbit', 'at'],
        typeCreators_1.createFunctionType('orbitat', orbitInfo_1.orbitInfoType, orbitable_1.orbitableType, timespan_1.timeSpanType),
    ],
    [['path'], typeCreators_1.createFunctionType('path', path_1.pathType, string_1.stringType)],
    [
        ['pid', 'loop'],
        typeCreators_1.createFunctionType('pidloop', pidLoop_1.pidLoopType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType),
    ],
    [
        ['position', 'at'],
        typeCreators_1.createFunctionType('positionat', vector_1.vectorType, orbitable_1.orbitableType, timespan_1.timeSpanType),
    ],
    [['print'], typeCreators_1.createFunctionType('print', void_1.voidType, structure_1.structureType)],
    [
        ['print', 'at'],
        typeCreators_1.createFunctionType('printat', structure_1.structureType, scalar_1.scalarType, scalar_1.scalarType),
    ],
    [['print', 'list'], typeCreators_1.createFunctionType('printlist', void_1.voidType, string_1.stringType)],
    [
        ['processor'],
        typeCreators_1.createFunctionType('processor', part_1.partType, /* TODO Union Type */ string_1.stringType),
    ],
    [['profile', 'result'], typeCreators_1.createFunctionType('profileresult', void_1.voidType)],
    [
        ['q'],
        typeCreators_1.createFunctionType('q', direction_1.directionType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType),
    ],
    [
        ['queue'],
        typeCreators_1.createVarFunctionType('queue', queue_1.queueType.toConcreteType(structure_1.structureType), typeCreators_1.createVarType(structure_1.structureType)),
    ],
    [
        ['r'],
        typeCreators_1.createFunctionType('r', direction_1.directionType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType),
    ],
    [['random'], typeCreators_1.createFunctionType('random', scalar_1.scalarType)],
    [
        ['range'],
        typeCreators_1.createFunctionType('range', range_1.rangeType, scalar_1.integarType, scalar_1.integarType, scalar_1.integarType),
    ],
    [
        ['read', 'json'],
        typeCreators_1.createFunctionType('readjson', serializeableStructure_1.serializableStructureType, string_1.stringType),
    ],
    [['reboot'], typeCreators_1.createFunctionType('reboot', void_1.voidType)],
    [['remove'], typeCreators_1.createFunctionType('remove', void_1.voidType, node_1.nodeType)],
    [
        ['rgb'],
        typeCreators_1.createFunctionType('rgb', rgba_1.rgbaType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType),
    ],
    [
        ['rgba'],
        typeCreators_1.createFunctionType('rgba', rgba_1.rgbaType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType),
    ],
    [
        ['rotate', 'from', 'to'],
        typeCreators_1.createFunctionType('rotatefromto', direction_1.directionType, vector_1.vectorType, vector_1.vectorType),
    ],
    [['round'], typeCreators_1.createFunctionType('round', scalar_1.scalarType, scalar_1.scalarType)],
    [['script', 'path'], typeCreators_1.createFunctionType('scriptpath', path_1.pathType)],
    [
        ['select', 'auto', 'pilot', 'mode'],
        typeCreators_1.createFunctionType('selectautopilotmode', void_1.voidType, string_1.stringType),
    ],
    [['shutdown'], typeCreators_1.createFunctionType('shutdown', void_1.voidType)],
    [['sin'], typeCreators_1.createFunctionType('sin', scalar_1.scalarType)],
    [
        ['slide', 'note'],
        typeCreators_1.createFunctionType('slidenote', note_1.noteType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType),
    ],
    [['sqrt'], typeCreators_1.createFunctionType('sqrt', scalar_1.scalarType, scalar_1.scalarType)],
    [
        ['stack'],
        typeCreators_1.createVarFunctionType('stack', stack_1.stackType.toConcreteType(structure_1.structureType), typeCreators_1.createVarType(structure_1.structureType)),
    ],
    [['stage'], typeCreators_1.createFunctionType('stage', void_1.voidType)],
    [['stop', 'all', 'voices'], typeCreators_1.createFunctionType('stopallvoices', void_1.voidType)],
    [['switch'], typeCreators_1.createFunctionType('switch', string_1.stringType)],
    [['tan'], typeCreators_1.createFunctionType('tan', scalar_1.scalarType)],
    [
        ['toggle', 'fly', 'by', 'wire'],
        typeCreators_1.createFunctionType('toggleflybywire', void_1.voidType, string_1.stringType, boolean_1.booleanType),
    ],
    [
        ['transfer'],
        typeCreators_1.createFunctionType('transfer', resourceTransfer_1.resourceTransferType, string_1.stringType, structure_1.structureType, structure_1.structureType, scalar_1.scalarType),
    ],
    [
        ['transfer', 'all'],
        typeCreators_1.createFunctionType('transferall', resourceTransfer_1.resourceTransferType, string_1.stringType, structure_1.structureType, structure_1.structureType),
    ],
    [['unchar'], typeCreators_1.createFunctionType('unchar', scalar_1.scalarType, string_1.stringType)],
    [
        ['unique', 'set'],
        typeCreators_1.createVarFunctionType('uniqueset', uniqueset_1.uniqueSetType.toConcreteType(structure_1.structureType), typeCreators_1.createVarType(structure_1.structureType)),
    ],
    [
        ['v'],
        typeCreators_1.createFunctionType('v', vector_1.vectorType, scalar_1.scalarType, scalar_1.scalarType, scalar_1.scalarType),
    ],
    [
        ['v', 'ang'],
        typeCreators_1.createFunctionType('vang', vector_1.vectorType, vector_1.vectorType, vector_1.vectorType),
    ],
    [
        ['v', 'crs'],
        typeCreators_1.createFunctionType('vcrs', vector_1.vectorType, vector_1.vectorType, vector_1.vectorType),
    ],
    [
        ['v', 'dot'],
        typeCreators_1.createFunctionType('vdot', vector_1.vectorType, vector_1.vectorType, vector_1.vectorType),
    ],
    [
        ['vec', 'draw'],
        typeCreators_1.createFunctionType('vecdraw', vectorRenderer_1.vectorRendererType, vector_1.vectorType, vector_1.vectorType, rgba_1.rgbaType, string_1.stringType, scalar_1.scalarType, boolean_1.booleanType, scalar_1.scalarType),
    ],
    [
        ['vec', 'draw', 'args'],
        typeCreators_1.createFunctionType('vecdrawargs', vectorRenderer_1.vectorRendererType, vector_1.vectorType, vector_1.vectorType, rgba_1.rgbaType, string_1.stringType, scalar_1.scalarType, boolean_1.booleanType, scalar_1.scalarType),
    ],
    [
        ['vector', 'angle'],
        typeCreators_1.createFunctionType('vectorangle', scalar_1.scalarType, vector_1.vectorType, vector_1.vectorType),
    ],
    [
        ['vector', 'cross', 'product'],
        typeCreators_1.createFunctionType('vectorcrossproduct', vector_1.vectorType, vector_1.vectorType, vector_1.vectorType),
    ],
    [
        ['vector', 'dot', 'product'],
        typeCreators_1.createFunctionType('vectordotproduct', vector_1.vectorType, vector_1.vectorType, vector_1.vectorType),
    ],
    [
        ['vector', 'exclude'],
        typeCreators_1.createFunctionType('vectorexclude', vector_1.vectorType, vector_1.vectorType, vector_1.vectorType),
    ],
    [
        ['velocity', 'at'],
        typeCreators_1.createFunctionType('velocityat', vector_1.vectorType, orbitable_1.orbitableType, timespan_1.timeSpanType),
    ],
    [['vessel'], typeCreators_1.createFunctionType('vessel', vesselTarget_1.vesselTargetType, string_1.stringType)],
    [['volume'], typeCreators_1.createFunctionType('volume', volume_1.volumeType, string_1.stringType)],
    [['vxcl'], typeCreators_1.createFunctionType('vxcl', vector_1.vectorType, vector_1.vectorType, vector_1.vectorType)],
    [['warp', 'to'], typeCreators_1.createFunctionType('warpto', void_1.voidType, scalar_1.scalarType)],
    [['waypoint'], typeCreators_1.createFunctionType('waypoint', waypoint_1.waypointType, string_1.stringType)],
    // TODO Union Types
    [
        ['write', 'json'],
        typeCreators_1.createFunctionType('writejson', volumneFile_1.volumeFileType, serializeableStructure_1.serializableStructureType, string_1.stringType),
    ],
];
// createFunctionType('rename_file_deprecated', /* TODO */ scalarType),
// createFunctionType('rename_volume_deprecated', /* TODO */ scalarType),
// createFunctionType('copy_deprecated', /* TODO */ scalarType),
// createFunctionType('delete_deprecated', /* TODO */ scalarType),
// createFunctionType('run', /* TODO */ scalarType),
const locks = [
    [['throttle'], scalar_1.scalarType],
    [['steering'], direction_1.directionType],
    [['wheel', 'throttle'], scalar_1.scalarType],
    [['wheel', 'steering'], direction_1.directionType],
    [['sas', 'mode'], string_1.stringType],
    [['nav', 'mode'], string_1.stringType],
];
const variables = [
    [['abort'], boolean_1.booleanType],
    [['active', 'ship'], vesselTarget_1.vesselTargetType],
    [['addons'], addonList_1.addonListType],
    [['ag1'], boolean_1.booleanType],
    [['ag10'], boolean_1.booleanType],
    [['ag2'], boolean_1.booleanType],
    [['ag3'], boolean_1.booleanType],
    [['ag4'], boolean_1.booleanType],
    [['ag5'], boolean_1.booleanType],
    [['ag6'], boolean_1.booleanType],
    [['ag7'], boolean_1.booleanType],
    [['ag8'], boolean_1.booleanType],
    [['ag9'], boolean_1.booleanType],
    [['airspeed'], scalar_1.scalarType],
    [['all', 'nodes'], list_1.listType.toConcreteType(node_1.nodeType)],
    [['alt'], vesselAlt_1.vesselAltType],
    [['altitude'], scalar_1.scalarType],
    [['angular', 'momentum'], vector_1.vectorType],
    [['angular', 'vel'], vector_1.vectorType],
    [['angular', 'velocity'], vector_1.vectorType],
    [['apoapsis'], scalar_1.scalarType],
    [['archive'], volume_1.volumeType],
    [['available', 'thrust'], scalar_1.scalarType],
    [['bays'], boolean_1.booleanType],
    [['black'], rgba_1.rgbaType],
    [['blue'], rgba_1.rgbaType],
    [['body'], bodyTarget_1.bodyTargetType],
    [['brakes'], boolean_1.booleanType],
    [['chutes'], boolean_1.booleanType],
    [['chutes', 'safe'], boolean_1.booleanType],
    [['config'], config_1.configType],
    [['constant'], constant_1.constantType],
    [['control', 'connection'], controlConnection_1.controlConnectionType],
    [['core'], core_1.coreType],
    [['cyan'], rgba_1.rgbaType],
    [['deploy', 'drills'], boolean_1.booleanType],
    [['donothing'], delegate_1.delegateType],
    [['drills'], boolean_1.booleanType],
    [['encounter'], orbitInfo_1.orbitInfoType],
    [['eta'], vesselEta_1.vesselEtaType],
    [['facing'], direction_1.directionType],
    [['fuel', 'cells'], boolean_1.booleanType],
    [['gear'], boolean_1.booleanType],
    [['geo', 'position'], geoCoordinates_1.geoCoordinatesType],
    [['gray'], rgba_1.rgbaType],
    [['green'], rgba_1.rgbaType],
    [['grey'], rgba_1.rgbaType],
    [['has', 'node'], boolean_1.booleanType],
    [['has', 'target'], boolean_1.booleanType],
    [['heading'], scalar_1.scalarType],
    [['home', 'connection'], homeConnection_1.homeConnectionType],
    [['intakes'], boolean_1.booleanType],
    [['isru'], boolean_1.booleanType],
    [['kuniverse'], kUniverse_1.kUniverseType],
    [['ladders'], boolean_1.booleanType],
    [['latitude'], scalar_1.scalarType],
    [['legs'], boolean_1.booleanType],
    [['lights'], boolean_1.booleanType],
    [['longitude'], scalar_1.scalarType],
    [['magenta'], rgba_1.rgbaType],
    [['map', 'view'], boolean_1.booleanType],
    [['mass'], scalar_1.scalarType],
    [['max', 'thrust'], scalar_1.scalarType],
    [['mission', 'time'], scalar_1.scalarType],
    [['next', 'node'], node_1.nodeType],
    [['north'], direction_1.directionType],
    [['obt'], orbitInfo_1.orbitInfoType],
    [['orbit'], orbitInfo_1.orbitInfoType],
    [['panels'], boolean_1.booleanType],
    [['periapsis'], scalar_1.scalarType],
    [['prograde'], direction_1.directionType],
    [['purple'], rgba_1.rgbaType],
    [['radiators'], boolean_1.booleanType],
    [['rcs'], boolean_1.booleanType],
    [['red'], rgba_1.rgbaType],
    [['retrograde'], direction_1.directionType],
    [['sas'], boolean_1.booleanType],
    [['sensor'], vesselSensors_1.vesselSensorsType],
    [['session', 'time'], scalar_1.doubleType],
    [['ship'], vesselTarget_1.vesselTargetType],
    [['ship', 'name'], string_1.stringType],
    [['solar', 'prime', 'vector'], vector_1.vectorType],
    [['srf', 'prograde'], direction_1.directionType],
    [['srf', 'retrograde'], direction_1.directionType],
    [['stage'], stage_1.stageType],
    [['status'], string_1.stringType],
    [['steering', 'manager'], steeringManager_1.steeringManagerType],
    [['surface', 'speed'], scalar_1.scalarType],
    [['target'], orbitable_1.orbitableType],
    [['terminal'], terminalStruct_1.terminalStructType],
    [['time'], timespan_1.timeSpanType],
    [['up'], direction_1.directionType],
    [['velocity'], orbitable_1.orbitableType],
    [['version'], versionInfo_1.versionInfoType],
    [['vertical', 'speed'], scalar_1.scalarType],
    [['volume:name'], string_1.stringType],
    [['warp'], scalar_1.integarType],
    [['warpmode'], string_1.stringType],
    [['white'], rgba_1.rgbaType],
    [['yellow'], rgba_1.rgbaType],
];
const bodies = [
    ['kerbol', bodyTarget_1.bodyTargetType],
    ['moho', bodyTarget_1.bodyTargetType],
    ['eve', bodyTarget_1.bodyTargetType],
    ['gilly', bodyTarget_1.bodyTargetType],
    ['kerbin', bodyTarget_1.bodyTargetType],
    ['mun', bodyTarget_1.bodyTargetType],
    ['minmus', bodyTarget_1.bodyTargetType],
    ['duna', bodyTarget_1.bodyTargetType],
    ['ike', bodyTarget_1.bodyTargetType],
    ['dres', bodyTarget_1.bodyTargetType],
    ['jool', bodyTarget_1.bodyTargetType],
    ['laythe', bodyTarget_1.bodyTargetType],
    ['vall', bodyTarget_1.bodyTargetType],
    ['tylo', bodyTarget_1.bodyTargetType],
    ['bop', bodyTarget_1.bodyTargetType],
    ['pol', bodyTarget_1.bodyTargetType],
    ['eeloo', bodyTarget_1.bodyTargetType],
];
// obsoleted
//  ['groundspeed', scalarType],
exports.standardLibraryBuilder = (caseKind) => {
    const libraryBuilder = new symbolTableBuilder_1.SymbolTableBuilder(constants_1.builtIn);
    for (const [segements, functionType] of functionTypes) {
        libraryBuilder.declareFunction(types_1.ScopeKind.global, new token_1.Token(tokentypes_1.TokenType.identifier, stringUtils_1.toCase(caseKind, ...segements), undefined, new marker_1.Marker(0, 0), new marker_1.Marker(0, 0), constants_1.builtIn), Array.isArray(functionType.params) ? functionType.params.length : -1, 0, false, functionType);
    }
    for (const [segements, type] of variables) {
        libraryBuilder.declareVariable(types_1.ScopeKind.global, new token_1.Token(tokentypes_1.TokenType.identifier, stringUtils_1.toCase(caseKind, ...segements), undefined, new marker_1.Marker(0, 0), new marker_1.Marker(0, 0), constants_1.builtIn), type);
    }
    for (const [segment, type] of locks) {
        libraryBuilder.declareLock(types_1.ScopeKind.global, new token_1.Token(tokentypes_1.TokenType.identifier, stringUtils_1.toCase(caseKind, ...segment), undefined, new marker_1.Marker(0, 0), new marker_1.Marker(0, 0), constants_1.builtIn), type);
    }
    return libraryBuilder.build();
};
exports.bodyLibraryBuilder = (caseKind) => {
    const bodyBuilder = new symbolTableBuilder_1.SymbolTableBuilder(constants_1.builtIn);
    for (const [identifier, type] of bodies) {
        bodyBuilder.declareVariable(types_1.ScopeKind.global, new token_1.Token(tokentypes_1.TokenType.identifier, stringUtils_1.toCase(caseKind, identifier), undefined, new marker_1.Marker(0, 0), new marker_1.Marker(0, 0), constants_1.builtIn), type);
    }
    return bodyBuilder.build();
};
//# sourceMappingURL=standardLibrary.js.map