import { SymbolTrackerBase, KsSet, IKsDeclared, TrackerKind, KsBaseSymbol } from './types';
import { Location } from 'vscode-languageserver';
import { ArgumentType, IFunctionType } from '../typeChecker/types/types';
import { FunctionType } from '../typeChecker/ksType';
export declare class BasicTracker<T extends KsBaseSymbol = KsBaseSymbol> implements SymbolTrackerBase {
    uri: string;
    type: ArgumentType | IFunctionType;
    readonly declared: IKsDeclared<T, ArgumentType | IFunctionType>;
    /**
     * Infromation about locations where the this symbol was set
     */
    readonly sets: KsSet[];
    /**
     * Locations where this symbol was used
     */
    readonly usages: Location[];
    constructor(symbol: T, uri: string, type?: ArgumentType | IFunctionType);
    /**
     * What kind of tracker is this type
     */
    readonly kind: TrackerKind.basic;
    /**
     * Set the declared type of this symbol
     * @param type type to declare this symbol
     */
    declareType(type: ArgumentType | FunctionType): void;
    /**
     * Get the type at a provided location
     * @param loc location to check
     */
    private getLocation;
    /**
     * Get the type at a location
     * @param loc query location
     */
    getType(loc: Location): Maybe<ArgumentType | IFunctionType>;
    /**
     * Set the type at a location
     * @param loc location to set
     * @param type type to set
     */
    setType(loc: Location, type: ArgumentType | FunctionType): void;
}
/**
 * Create a set symbol
 * @param loc location to create the set
 * @param type type of this set
 */
export declare const createSymbolSet: (loc: Location, type?: import("../typeChecker/types/types").IBasicType) => KsSet;
