"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const structure_1 = require("../typeChecker/types/primitives/structure");
const positionUtils_1 = require("../utilities/positionUtils");
const constants_1 = require("../utilities/constants");
class BasicTracker {
    constructor(symbol, uri, type = structure_1.structureType) {
        this.uri = uri;
        this.type = type;
        this.declared = {
            symbol,
            type,
            uri: symbol.name.uri,
            range: symbol.name.range,
        };
        this.sets = [];
        this.usages = [];
    }
    /**
     * What kind of tracker is this type
     */
    get kind() {
        return 0 /* basic */;
    }
    /**
     * Set the declared type of this symbol
     * @param type type to declare this symbol
     */
    declareType(type) {
        this.declared.type = type;
    }
    /**
     * Get the type at a provided location
     * @param loc location to check
     */
    getLocation(loc) {
        // if builtin type can't change so just return declared type
        if (this.declared.uri === constants_1.builtIn) {
            return this.declared;
        }
        // Will need ot do better this sort of handles the case of
        // local x is 0. set x to x + 1.
        const ranges = this.sets.filter(set => set.uri === loc.uri && set.type !== structure_1.structureType);
        const found = positionUtils_1.binaryRightKey(ranges, loc.range.start, x => x.range);
        return found || this.declared;
    }
    /**
     * Get the type at a location
     * @param loc query location
     */
    getType(loc) {
        return this.getLocation(loc).type;
    }
    /**
     * Set the type at a location
     * @param loc location to set
     * @param type type to set
     */
    setType(loc, type) {
        const nearestSet = this.getLocation(loc);
        if (positionUtils_1.locationEqual(nearestSet, loc)) {
            nearestSet.type = type;
        }
    }
}
exports.BasicTracker = BasicTracker;
/**
 * Create a set symbol
 * @param loc location to create the set
 * @param type type of this set
 */
exports.createSymbolSet = (loc, type = structure_1.structureType) => ({ type, uri: loc.uri, range: loc.range });
//# sourceMappingURL=tracker.js.map