"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const scanner_1 = require("../scanner/scanner");
const fsUtils_1 = require("../utilities/fsUtils");
const fs_1 = require("fs");
const path_1 = require("path");
const perf_hooks_1 = require("perf_hooks");
const testDir = path_1.join(__dirname, '../../../kerboscripts/parser_valid/');
fsUtils_1.walkDir(testDir, (filePath) => {
    const kosFile = fs_1.readFileSync(filePath, 'utf8');
    const scanner = new scanner_1.Scanner(kosFile);
    scanner.scanTokens();
});
let results = [];
for (let i = 0; i < 10; i += 1) {
    fsUtils_1.walkDir(testDir, (filePath) => {
        const size = fs_1.statSync(filePath).size;
        const kosFile = fs_1.readFileSync(filePath, 'utf8');
        const start = perf_hooks_1.performance.now();
        const scanner = new scanner_1.Scanner(kosFile);
        scanner.scanTokens();
        const end = perf_hooks_1.performance.now();
        results.push({
            filePath,
            size: size / 1024,
            time: end - start,
            rate: size / ((end - start) / 1000 * 1024),
        });
    });
}
results = results.sort((x, y) => (y.rate - x.rate));
for (const result of results) {
    // tslint:disable-next-line:max-line-length
    console.log(`file path: ${result.filePath} size: ${result.size.toFixed(1)} KB, time ${result.time.toFixed(2)} ms ${result.rate.toFixed(2)} KB/s`);
}
let time = 0;
let size = 0;
for (const result of results) {
    time += result.time;
    size += result.size;
}
// tslint:disable-next-line:max-line-length
console.log(`Overall rate ${size / (time / 1000)} KB/s`);
//# sourceMappingURL=scannerBench.js.map