import { Stmt, Block } from './stmt';
import { IExpr, IStmtVisitor, ScopeKind, IStmtPasser, NodeDataBuilder } from './types';
import { Range, Position } from 'vscode-languageserver';
import { NodeBase } from './base';
import { Token } from '../entities/token';
export declare abstract class Decl extends Stmt {
    constructor();
}
export declare class Scope extends NodeBase {
    readonly scope?: Token | undefined;
    readonly declare?: Token | undefined;
    constructor(scope?: Token | undefined, declare?: Token | undefined);
    toLines(): string[];
    readonly ranges: Range[];
    readonly start: Position;
    readonly end: Position;
    readonly type: ScopeKind;
}
export declare class Var extends Decl {
    readonly identifier: Token;
    readonly toIs: Token;
    readonly value: IExpr;
    readonly scope: Scope;
    constructor(builder: NodeDataBuilder<Var>);
    toLines(): string[];
    readonly start: Position;
    readonly end: Position;
    readonly ranges: Range[];
    pass<T>(visitor: IStmtPasser<T>): T;
    accept<T>(visitor: IStmtVisitor<T>): T;
}
export declare class Lock extends Decl {
    readonly lock: Token;
    readonly identifier: Token;
    readonly to: Token;
    readonly value: IExpr;
    readonly scope?: Scope;
    constructor(builder: NodeDataBuilder<Lock>);
    toLines(): string[];
    readonly start: Position;
    readonly end: Position;
    readonly ranges: Range[];
    pass<T>(visitor: IStmtPasser<T>): T;
    accept<T>(visitor: IStmtVisitor<T>): T;
}
export declare class Func extends Decl {
    readonly functionToken: Token;
    readonly identifier: Token;
    readonly block: Block;
    readonly scope?: Scope;
    constructor(builder: NodeDataBuilder<Func>);
    toLines(): string[];
    readonly start: Position;
    readonly end: Position;
    readonly ranges: Range[];
    pass<T>(visitor: IStmtPasser<T>): T;
    accept<T>(visitor: IStmtVisitor<T>): T;
}
export declare class Param extends Decl {
    readonly parameterToken: Token;
    readonly requiredParameters: Parameter[];
    readonly optionalParameters: DefaultParam[];
    readonly scope?: Scope;
    constructor(builder: NodeDataBuilder<Param>);
    toLines(): string[];
    readonly start: Position;
    readonly end: Position;
    readonly ranges: Range[];
    pass<T>(visitor: IStmtPasser<T>): T;
    accept<T>(visitor: IStmtVisitor<T>): T;
}
export declare class Parameter extends NodeBase {
    readonly identifier: Token;
    constructor(identifier: Token);
    toLines(): string[];
    readonly start: Position;
    readonly end: Position;
    readonly ranges: Range[];
    readonly isKeyword: boolean;
}
export declare class DefaultParam extends Parameter {
    readonly toIs: Token;
    readonly value: IExpr;
    constructor(identifier: Token, toIs: Token, value: IExpr);
    toLines(): string[];
    readonly start: Position;
    readonly end: Position;
    readonly ranges: Range[];
    readonly isKeyword: boolean;
}
