"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stmt_1 = require("./stmt");
const types_1 = require("./types");
const tokentypes_1 = require("../entities/tokentypes");
const typeGuards_1 = require("../utilities/typeGuards");
const toStringUtils_1 = require("./toStringUtils");
const base_1 = require("./base");
class Decl extends stmt_1.Stmt {
    constructor() {
        super();
    }
}
exports.Decl = Decl;
class Scope extends base_1.NodeBase {
    constructor(scope, declare) {
        super();
        this.scope = scope;
        this.declare = declare;
    }
    toLines() {
        if (!typeGuards_1.empty(this.scope) && !typeGuards_1.empty(this.declare)) {
            return [`${this.declare.lexeme} ${this.scope.lexeme}`];
        }
        if (!typeGuards_1.empty(this.scope)) {
            return [this.scope.lexeme];
        }
        if (!typeGuards_1.empty(this.declare)) {
            return [this.declare.lexeme];
        }
        throw new Error('Unvalid scope encountered. No socpe or declare tokens');
    }
    get ranges() {
        if (!typeGuards_1.empty(this.scope) && !typeGuards_1.empty(this.declare)) {
            return [this.declare, this.scope];
        }
        if (!typeGuards_1.empty(this.scope)) {
            return [this.scope];
        }
        if (!typeGuards_1.empty(this.declare)) {
            return [this.declare];
        }
        throw new Error('Unvalid scope encountered. No socpe or declare tokens');
    }
    get start() {
        if (!typeGuards_1.empty(this.scope) && !typeGuards_1.empty(this.declare)) {
            return this.declare.start;
        }
        if (!typeGuards_1.empty(this.scope)) {
            return this.scope.start;
        }
        if (!typeGuards_1.empty(this.declare)) {
            return this.declare.start;
        }
        throw new Error('Unvalid scope encountered. No socpe or declare tokens');
    }
    get end() {
        if (!typeGuards_1.empty(this.scope) && !typeGuards_1.empty(this.declare)) {
            return this.scope.end;
        }
        if (!typeGuards_1.empty(this.scope)) {
            return this.scope.end;
        }
        if (!typeGuards_1.empty(this.declare)) {
            return this.declare.end;
        }
        throw new Error('Unvalid scope encountered. No socpe or declare tokens');
    }
    get type() {
        if (typeGuards_1.empty(this.scope)) {
            return types_1.ScopeKind.local;
        }
        switch (this.scope.type) {
            case tokentypes_1.TokenType.local:
                return types_1.ScopeKind.local;
            case tokentypes_1.TokenType.global:
                return types_1.ScopeKind.global;
            default:
                throw new Error('Unknown scope type found');
        }
    }
}
exports.Scope = Scope;
class Var extends Decl {
    constructor(builder) {
        super();
        this.identifier = typeGuards_1.unWrap(builder.identifier);
        this.toIs = typeGuards_1.unWrap(builder.toIs);
        this.value = typeGuards_1.unWrap(builder.value);
        this.scope = typeGuards_1.unWrap(builder.scope);
    }
    toLines() {
        const lines = this.value.toLines();
        lines[0] =
            `${this.scope.toString()} ${this.identifier.lexeme} ` +
                `${this.toIs.lexeme} ${lines[0]}`;
        lines[lines.length - 1] = `${lines[lines.length - 1]}.`;
        return lines;
    }
    get start() {
        return this.scope.start;
    }
    get end() {
        return this.value.end;
    }
    get ranges() {
        return [this.scope, this.identifier, this.toIs, this.value];
    }
    pass(visitor) {
        return visitor.passDeclVariable(this);
    }
    accept(visitor) {
        return visitor.visitDeclVariable(this);
    }
}
exports.Var = Var;
class Lock extends Decl {
    constructor(builder) {
        super();
        this.lock = typeGuards_1.unWrap(builder.lock);
        this.identifier = typeGuards_1.unWrap(builder.identifier);
        this.to = typeGuards_1.unWrap(builder.to);
        this.value = typeGuards_1.unWrap(builder.value);
        this.scope = builder.scope;
    }
    toLines() {
        const lines = this.value.toLines();
        if (typeGuards_1.empty(this.scope)) {
            lines[0] =
                `${this.lock.lexeme} ${this.identifier.lexeme} ` +
                    `${this.to.lexeme} ${lines[0]}`;
        }
        else {
            lines[0] =
                `${this.scope.toString()} ${this.lock.lexeme} ${this.identifier.lexeme} ` + `${this.to.lexeme} ${lines[0]}`;
        }
        lines[lines.length - 1] = `${lines[lines.length - 1]}.`;
        return lines;
    }
    get start() {
        return typeGuards_1.empty(this.scope) ? this.lock.start : this.scope.start;
    }
    get end() {
        return this.value.end;
    }
    get ranges() {
        if (!typeGuards_1.empty(this.scope)) {
            return [this.scope, this.lock, this.identifier, this.to, this.value];
        }
        return [this.lock, this.identifier, this.to, this.value];
    }
    pass(visitor) {
        return visitor.passDeclLock(this);
    }
    accept(visitor) {
        return visitor.visitDeclLock(this);
    }
}
exports.Lock = Lock;
class Func extends Decl {
    constructor(builder) {
        super();
        this.functionToken = typeGuards_1.unWrap(builder.functionToken);
        this.identifier = typeGuards_1.unWrap(builder.identifier);
        this.block = typeGuards_1.unWrap(builder.block);
        this.scope = builder.scope;
    }
    toLines() {
        const declareLine = typeGuards_1.empty(this.scope)
            ? `${this.functionToken.lexeme} ${this.identifier.lexeme}`
            : `${this.scope.toString()} ${this.functionToken.lexeme} ${this.identifier.lexeme}`;
        const blockLines = this.block.toLines();
        return toStringUtils_1.joinLines(' ', [declareLine], blockLines);
    }
    get start() {
        return typeGuards_1.empty(this.scope) ? this.functionToken.start : this.scope.start;
    }
    get end() {
        return this.block.end;
    }
    get ranges() {
        if (!typeGuards_1.empty(this.scope)) {
            return [this.scope, this.functionToken, this.identifier, this.block];
        }
        return [this.functionToken, this.identifier, this.block];
    }
    pass(visitor) {
        return visitor.passDeclFunction(this);
    }
    accept(visitor) {
        return visitor.visitDeclFunction(this);
    }
}
exports.Func = Func;
class Param extends Decl {
    constructor(builder) {
        super();
        this.parameterToken = typeGuards_1.unWrap(builder.parameterToken);
        this.requiredParameters = typeGuards_1.unWrap(builder.requiredParameters);
        this.optionalParameters = typeGuards_1.unWrap(builder.optionalParameters);
        this.scope = builder.scope;
    }
    toLines() {
        const declareLine = typeGuards_1.empty(this.scope)
            ? [`${this.parameterToken.lexeme}`]
            : [`${this.scope.toString()} ${this.parameterToken.lexeme}`];
        const paramLines = toStringUtils_1.joinLines(', ', ...this.requiredParameters.map(param => param.toLines()));
        const defaultParamLines = toStringUtils_1.joinLines(', ', ...this.optionalParameters.map(param => param.toLines()));
        let lines = [];
        if (this.requiredParameters.length > 0 &&
            this.optionalParameters.length > 0) {
            lines = toStringUtils_1.joinLines(', ', paramLines, defaultParamLines);
        }
        else if (this.requiredParameters.length > 0) {
            lines = paramLines;
        }
        else {
            lines = defaultParamLines;
        }
        lines[0] = `${declareLine} ${lines[0]}`;
        lines[lines.length - 1] = `${lines[lines.length - 1]}.`;
        return lines;
    }
    get start() {
        return typeGuards_1.empty(this.scope) ? this.parameterToken.start : this.scope.start;
    }
    get end() {
        return this.optionalParameters.length > 0
            ? this.optionalParameters[this.optionalParameters.length - 1].value.end
            : this.requiredParameters[this.requiredParameters.length - 1].end;
    }
    get ranges() {
        if (!typeGuards_1.empty(this.scope)) {
            return [
                this.scope,
                this.parameterToken,
                ...this.requiredParameters,
                ...this.optionalParameters,
            ];
        }
        return [
            this.parameterToken,
            ...this.requiredParameters,
            ...this.optionalParameters,
        ];
    }
    pass(visitor) {
        return visitor.passDeclParameter(this);
    }
    accept(visitor) {
        return visitor.visitDeclParameter(this);
    }
}
exports.Param = Param;
class Parameter extends base_1.NodeBase {
    constructor(identifier) {
        super();
        this.identifier = identifier;
    }
    toLines() {
        return [this.identifier.lexeme];
    }
    get start() {
        return this.identifier.start;
    }
    get end() {
        return this.identifier.end;
    }
    get ranges() {
        return [this.identifier];
    }
    get isKeyword() {
        return this.identifier.type !== tokentypes_1.TokenType.identifier;
    }
}
exports.Parameter = Parameter;
class DefaultParam extends Parameter {
    constructor(identifier, toIs, value) {
        super(identifier);
        this.toIs = toIs;
        this.value = value;
    }
    toLines() {
        const lines = this.value.toLines();
        lines[0] = `${this.identifier.lexeme} ${this.toIs.lexeme} ${lines[0]}`;
        return lines;
    }
    get start() {
        return this.identifier.start;
    }
    get end() {
        return this.value.end;
    }
    get ranges() {
        return [this.identifier, this.toIs, this.value];
    }
    get isKeyword() {
        return this.identifier.type !== tokentypes_1.TokenType.identifier;
    }
}
exports.DefaultParam = DefaultParam;
//# sourceMappingURL=declare.js.map