import { GrammarNode, IGrammarOptional, IGrammarUnion, IGrammarRepeat, Distribution, INormalDistribution, IGammaDistribution, IExponentialDistribution, IConstantDistribution } from './types';
/**
 * Create an optional grammar node
 * @param dist a sampling distribution for when the node is included
 * @param nodes optional nodes
 */
export declare const createGrammarOptional: (dist: Distribution, ...nodes: GrammarNode[]) => IGrammarOptional;
/**
 * Create a repeating grammar node
 * @param dist a sampling distribution for how many times a node is included
 * @param nodes repeating nodes
 */
export declare const createGrammarRepeat: (dist: Distribution, ...nodes: GrammarNode[]) => IGrammarRepeat;
/**
 * Create a union grammar node
 * @param node a set of nodes and distribution for node selection
 */
export declare const createGrammarUnion: (...node: [GrammarNode, Distribution][]) => IGrammarUnion;
/**
 * Create a normal distribution
 * @param mean mean of the normal
 * @param std standard deviation for the normal
 */
export declare const createNormal: (mean: number, std: number) => INormalDistribution;
/**
 * Create a gamma distribution
 * @param shape shape factor for the gamma
 * @param scale scale factor for the gamma
 */
export declare const createGamma: (shape: number, scale: number) => IGammaDistribution;
/**
 * Create a exponential distribution
 * @param rate rate for the exp distribution
 */
export declare const createExponential: (rate: number) => IExponentialDistribution;
/**
 * Create a constant distribution
 * @param value constant value of distribution
 */
export declare const createConstant: (value: number) => IConstantDistribution;
