"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Create an optional grammar node
 * @param dist a sampling distribution for when the node is included
 * @param nodes optional nodes
 */
exports.createGrammarOptional = (dist, ...nodes) => ({ dist, nodes, tag: 'optional' });
/**
 * Create a repeating grammar node
 * @param dist a sampling distribution for how many times a node is included
 * @param nodes repeating nodes
 */
exports.createGrammarRepeat = (dist, ...nodes) => ({ dist, nodes, tag: 'repeat' });
/**
 * Create a union grammar node
 * @param node a set of nodes and distribution for node selection
 */
exports.createGrammarUnion = (...node) => ({ node, tag: 'union' });
/**
 * Create a normal distribution
 * @param mean mean of the normal
 * @param std standard deviation for the normal
 */
exports.createNormal = (mean, std) => ({ mean, std, tag: 'normal' });
/**
 * Create a gamma distribution
 * @param shape shape factor for the gamma
 * @param scale scale factor for the gamma
 */
exports.createGamma = (shape, scale) => ({ shape, scale, tag: 'gamma' });
/**
 * Create a exponential distribution
 * @param rate rate for the exp distribution
 */
exports.createExponential = (rate) => ({ rate, tag: 'exp' });
/**
 * Create a constant distribution
 * @param value constant value of distribution
 */
exports.createConstant = (value) => ({ value, tag: 'constant' });
//# sourceMappingURL=grammarNodes.js.map