import { IExpr, IStmt, INodeResult, IParseResult } from './types';
import { Token } from '../entities/token';
export declare class Parser {
    private uri;
    private tokens;
    private current;
    private runStmts;
    private readonly andBind;
    private readonly eqaulityBind;
    private readonly comparisonBind;
    private readonly additionBind;
    private readonly multiplicationBind;
    private readonly unaryBind;
    private readonly logger;
    private readonly tracer;
    constructor(uri: string, tokens: Token[], logger?: ILogger, tracer?: ITracer);
    parse(): IParseResult;
    parseStatement(): INodeResult<IStmt>;
    parseExpression(): INodeResult<IExpr>;
    parseArgCount(): INodeResult<number>;
    private eof;
    private declaration;
    private define;
    private declareFunction;
    private declareParameter;
    private declareNormalParameters;
    private declaredDefaultedParameters;
    private declareLock;
    private declareVariable;
    private statement;
    private block;
    private identifierLedStatement;
    private onOff;
    private command;
    private commandExpression;
    /**
     * Parse unset statement
     */
    private unset;
    /**
     * Parse unlock statement
     */
    private unlock;
    /**
     * Parse the set statement
     */
    private set;
    /**
     * Parse lazy global statement
     */
    private lazyGlobal;
    /**
     * Parse if statement
     */
    private ifStmt;
    private until;
    private from;
    private when;
    private returnStmt;
    private breakStmt;
    private switchStmt;
    private forStmt;
    private on;
    private toggle;
    private wait;
    private log;
    private copy;
    private rename;
    private delete;
    /**
     * Parse run statement
     */
    private run;
    /**
     * Parse run path statement
     */
    private runPath;
    /**
     * Parse run path once statement
     */
    private runOncePath;
    /**
     * parse a compile statement
     */
    private compile;
    /**
     * Parse a list statement
     */
    private list;
    /**
     * Parse a print statement
     */
    private print;
    /**
     * Parse an expression
     * @param stmt the statment context
     */
    private expression;
    private or;
    private and;
    private equality;
    private comparison;
    private addition;
    private multiplication;
    private binaryExpression;
    private unary;
    private factor;
    private suffixCatch;
    private suffix;
    private suffixTerm;
    private functionTrailer;
    private partialArgumentsCount;
    private arguments;
    private arrayBracket;
    private arrayIndex;
    private lambda;
    private atom;
    private addRunStmts;
    /**
     * Check the the statement is terminated
     * @param failed failed constructor context
     * @param partialNode partially constructed node
     */
    private terminal;
    /**
     * Check for any valid identifier throws
     * error if incorrect token is found
     * @param message message if token is not found
     * @param failed failed construcotor
     * @param partialNode partially constructed node
     */
    private consumeIdentifierThrow;
    /**
     * Consume current token if it matches the supplied type
     * throw error if incorrect token is found
     * @param message error message
     * @param failed fail context
     * @param tokenType token type
     * @param partialNode partially constructed node
     */
    private consumeTokenThrow;
    /**
     * Consume current token if it matches one of the supplied types
     * throw error if incorrect token is found
     * @param message error message
     * @param failed fail context
     * @param tokenTypes token types
     * @param partialNode partially constructed node
     */
    private consumeTokensThrow;
    /**
     * Consume current token if it matches one of the supplied types
     * returns error if incorrect token is found
     * @param message error message
     * @param failed fail context
     * @param tokenTypes token types
     * @param partialNode partially constructed node
     */
    private consumeTokenReturn;
    private matchIdentifier;
    private matchToken;
    private matchTokens;
    private identifierCheck;
    private check;
    private checkNext;
    private advance;
    private backup;
    private isAtEnd;
    private peek;
    private peekNext;
    private previous;
    private error;
    private synchronize;
}
