"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Decl = __importStar(require("./declare"));
const Stmt = __importStar(require("./stmt"));
const Expr = __importStar(require("./expr"));
const SuffixTerm = __importStar(require("./suffixTerm"));
const positionUtils_1 = require("../utilities/positionUtils");
const typeGuards_1 = require("../utilities/typeGuards");
const token_1 = require("../entities/token");
const treeExecute_1 = require("./treeExecute");
class ScriptFind extends treeExecute_1.TreeExecute {
    constructor() {
        super();
        this.pos = {
            line: 0,
            character: 0,
        };
        this.contexts = [];
    }
    /**
     * Find a token within a tree node
     * @param syntaxNode node to search in
     * @param pos position to find
     * @param contexts list of contexts to captures
     */
    find(syntaxNode, pos, ...contexts) {
        this.pos = pos;
        this.contexts = contexts;
        return this.nodeAction(syntaxNode);
    }
    nodeAction(node) {
        const searchResult = positionUtils_1.binarySearch(node.ranges, this.pos);
        if (typeGuards_1.empty(searchResult)) {
            return searchResult;
        }
        // search expression if expression
        if (searchResult instanceof SuffixTerm.SuffixTermBase) {
            const findResult = this.suffixTermAction(searchResult);
            // add context if not set yet
            if (!typeGuards_1.empty(findResult) && this.addContext(findResult, node)) {
                findResult.node = node;
            }
            return findResult;
        }
        // search expression if expression
        if (searchResult instanceof Expr.Expr) {
            const findResult = this.exprAction(searchResult);
            // add context if not set yet
            if (!typeGuards_1.empty(findResult) && this.addContext(findResult, node)) {
                findResult.node = node;
            }
            return findResult;
        }
        // search statement if statement
        if (searchResult instanceof Stmt.Stmt) {
            const findResult = this.stmtAction(searchResult);
            // add context if not set yet
            if (!typeGuards_1.empty(findResult) && this.addContext(findResult, node)) {
                findResult.node = node;
            }
            return findResult;
        }
        // search an optional parameter
        if (searchResult instanceof Decl.DefaultParam) {
            if (positionUtils_1.rangeContainsPos(searchResult.identifier, this.pos)) {
                return {
                    token: searchResult.identifier,
                    node: this.isContext(node)
                        ? node
                        : this.isContext(searchResult)
                            ? searchResult
                            : undefined,
                };
            }
            if (positionUtils_1.rangeContainsPos(searchResult.value, this.pos)) {
                return this.exprAction(searchResult.value);
            }
            return undefined;
        }
        // search a normal parameter
        if (searchResult instanceof Decl.Parameter) {
            return {
                node: this.isContext(node)
                    ? node
                    : this.isContext(searchResult)
                        ? searchResult
                        : undefined,
                token: searchResult.identifier,
            };
        }
        // return result if token found
        if (searchResult instanceof token_1.Token) {
            return {
                node: this.isContext(node)
                    ? node
                    : undefined,
                token: searchResult,
            };
        }
        // return result if token found
        if (searchResult instanceof Decl.Scope) {
            if (!typeGuards_1.empty(searchResult.scope) && positionUtils_1.rangeContainsPos(searchResult.scope, this.pos)) {
                return {
                    node: undefined,
                    token: searchResult.scope,
                };
            }
            if (!typeGuards_1.empty(searchResult.declare) && positionUtils_1.rangeContainsPos(searchResult.declare, this.pos)) {
                return {
                    node: undefined,
                    token: searchResult.declare,
                };
            }
        }
        throw new Error('Unexpected result found.');
    }
    addContext(findResult, node) {
        return typeGuards_1.empty(findResult.node) && this.isContext(node);
    }
    // is the correct context
    isContext(node) {
        return this.contexts.some(context => node instanceof context);
    }
}
exports.ScriptFind = ScriptFind;
//# sourceMappingURL=scriptFind.js.map