"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Decl = __importStar(require("./declare"));
const Stmt = __importStar(require("./stmt"));
const Expr = __importStar(require("./expr"));
const SuffixTerm = __importStar(require("./suffixTerm"));
const token_1 = require("../entities/token");
const treeExecute_1 = require("./treeExecute");
const arrayUtils_1 = require("../utilities/arrayUtils");
/**
 * Check all tokens in a given tree node
 */
class TokenCheck extends treeExecute_1.TreeExecute {
    /**
     * Construct token check
     */
    constructor() {
        super();
    }
    /**
     * Get all tokens in the syntax tree from this node down
     * @param syntaxNode syntax node to begin
     */
    orderedTokens(syntaxNode) {
        return this.nodeAction(syntaxNode);
    }
    /**
     * Add tokens from each internal node
     * @param node syntax node to begin
     */
    nodeAction(node) {
        // return result if token found
        if (node instanceof token_1.Token) {
            return [node];
        }
        const tokens = [];
        for (const childNode of node.ranges) {
            if (childNode instanceof SuffixTerm.SuffixTermBase) {
                tokens.push(this.suffixTermAction(childNode));
            }
            else if (childNode instanceof Expr.Expr) {
                tokens.push(this.exprAction(childNode));
            }
            else if (childNode instanceof Stmt.Stmt) {
                tokens.push(this.stmtAction(childNode));
            }
            else if (childNode instanceof Decl.Param) {
                const params = [];
                for (const param of childNode.requiredParameters) {
                    params.push(param.identifier);
                }
                for (const param of childNode.optionalParameters) {
                    params.push(param.identifier);
                    params.push(param.toIs);
                    params.push(...this.exprAction(param.value));
                }
                tokens.push(params);
            }
        }
        return arrayUtils_1.flatten(tokens);
    }
}
exports.TokenCheck = TokenCheck;
//# sourceMappingURL=tokenCheck.js.map