import { IExprVisitor, IStmtVisitor, IStmt, IExpr, ScriptNode, ISuffixTermVisitor, ISuffixTerm } from './types';
import * as Decl from './declare';
import * as Stmt from './stmt';
import * as Expr from './expr';
import * as SuffixTerm from './suffixTerm';
export declare abstract class TreeExecute<T> implements IExprVisitor<T>, IStmtVisitor<T>, ISuffixTermVisitor<T> {
    constructor();
    protected abstract nodeAction(node: ScriptNode): T;
    protected stmtAction(stmt: IStmt): T;
    protected exprAction(expr: IExpr): T;
    protected suffixTermAction(suffixTerm: ISuffixTerm): T;
    visitDeclVariable(decl: Decl.Var): T;
    visitDeclLock(decl: Decl.Lock): T;
    visitDeclFunction(decl: Decl.Func): T;
    visitDeclParameter(decl: Decl.Param): T;
    visitStmtInvalid(stmt: Stmt.Invalid): T;
    visitBlock(stmt: Stmt.Block): T;
    visitExpr(stmt: Stmt.ExprStmt): T;
    visitOnOff(stmt: Stmt.OnOff): T;
    visitCommand(stmt: Stmt.Command): T;
    visitCommandExpr(stmt: Stmt.CommandExpr): T;
    visitUnset(stmt: Stmt.Unset): T;
    visitUnlock(stmt: Stmt.Unlock): T;
    visitSet(stmt: Stmt.Set): T;
    visitLazyGlobal(stmt: Stmt.LazyGlobal): T;
    visitIf(stmt: Stmt.If): T;
    visitElse(stmt: Stmt.Else): T;
    visitUntil(stmt: Stmt.Until): T;
    visitFrom(stmt: Stmt.From): T;
    visitWhen(stmt: Stmt.When): T;
    visitReturn(stmt: Stmt.Return): T;
    visitBreak(stmt: Stmt.Break): T;
    visitSwitch(stmt: Stmt.Switch): T;
    visitFor(stmt: Stmt.For): T;
    visitOn(stmt: Stmt.On): T;
    visitToggle(stmt: Stmt.Toggle): T;
    visitWait(stmt: Stmt.Wait): T;
    visitLog(stmt: Stmt.Log): T;
    visitCopy(stmt: Stmt.Copy): T;
    visitRename(stmt: Stmt.Rename): T;
    visitDelete(stmt: Stmt.Delete): T;
    visitRun(stmt: Stmt.Run): T;
    visitRunPath(stmt: Stmt.RunPath): T;
    visitRunPathOnce(stmt: Stmt.RunOncePath): T;
    visitCompile(stmt: Stmt.Compile): T;
    visitList(stmt: Stmt.List): T;
    visitEmpty(stmt: Stmt.Empty): T;
    visitPrint(stmt: Stmt.Print): T;
    visitExprInvalid(expr: Expr.Invalid): T;
    visitBinary(expr: Expr.Binary): T;
    visitUnary(expr: Expr.Unary): T;
    visitFactor(expr: Expr.Factor): T;
    visitSuffix(expr: Expr.Suffix): T;
    visitSuffixTermInvalid(suffixTerm: SuffixTerm.Invalid): T;
    visitSuffixTrailer(suffixTerm: SuffixTerm.SuffixTrailer): T;
    visitSuffixTerm(suffixTerm: SuffixTerm.SuffixTerm): T;
    visitCall(suffixTerm: SuffixTerm.Call): T;
    visitArrayIndex(suffixTerm: SuffixTerm.ArrayIndex): T;
    visitArrayBracket(suffixTerm: SuffixTerm.ArrayBracket): T;
    visitDelegate(suffixTerm: SuffixTerm.Delegate): T;
    visitLiteral(suffixTerm: SuffixTerm.Literal): T;
    visitIdentifier(suffixTerm: SuffixTerm.Identifier): T;
    visitGrouping(suffixTerm: SuffixTerm.Grouping): T;
    visitLambda(expr: Expr.Lambda): T;
}
