import { IExprVisitor, IStmtVisitor, IStmt, IExpr, ScriptNode, ISuffixTermVisitor, ISuffixTerm } from './types';
import * as Decl from './declare';
import * as Stmt from './stmt';
import * as Expr from './expr';
import * as SuffixTerm from './suffixTerm';
export declare abstract class TreeTraverse implements IExprVisitor<void>, IStmtVisitor<void>, ISuffixTermVisitor<void> {
    constructor();
    protected abstract nodeAction(node: ScriptNode): boolean;
    protected stmtAction(stmt: IStmt): void;
    protected exprAction(expr: IExpr): void;
    protected suffixTermAction(suffixTerm: ISuffixTerm): void;
    visitDeclVariable(decl: Decl.Var): void;
    visitDeclLock(decl: Decl.Lock): void;
    visitDeclFunction(decl: Decl.Func): void;
    visitDeclParameter(decl: Decl.Param): void;
    visitStmtInvalid(stmt: Stmt.Invalid): void;
    visitBlock(stmt: Stmt.Block): void;
    visitExpr(stmt: Stmt.ExprStmt): void;
    visitOnOff(stmt: Stmt.OnOff): void;
    visitCommand(stmt: Stmt.Command): void;
    visitCommandExpr(stmt: Stmt.CommandExpr): void;
    visitUnset(stmt: Stmt.Unset): void;
    visitUnlock(stmt: Stmt.Unlock): void;
    visitSet(stmt: Stmt.Set): void;
    visitLazyGlobal(stmt: Stmt.LazyGlobal): void;
    visitIf(stmt: Stmt.If): void;
    visitElse(stmt: Stmt.Else): void;
    visitUntil(stmt: Stmt.Until): void;
    visitFrom(stmt: Stmt.From): void;
    visitWhen(stmt: Stmt.When): void;
    visitReturn(stmt: Stmt.Return): void;
    visitBreak(stmt: Stmt.Break): void;
    visitSwitch(stmt: Stmt.Switch): void;
    visitFor(stmt: Stmt.For): void;
    visitOn(stmt: Stmt.On): void;
    visitToggle(stmt: Stmt.Toggle): void;
    visitWait(stmt: Stmt.Wait): void;
    visitLog(stmt: Stmt.Log): void;
    visitCopy(stmt: Stmt.Copy): void;
    visitRename(stmt: Stmt.Rename): void;
    visitDelete(stmt: Stmt.Delete): void;
    visitRun(stmt: Stmt.Run): void;
    visitRunPath(stmt: Stmt.RunPath): void;
    visitRunPathOnce(stmt: Stmt.RunOncePath): void;
    visitCompile(stmt: Stmt.Compile): void;
    visitList(stmt: Stmt.List): void;
    visitEmpty(stmt: Stmt.Empty): void;
    visitPrint(stmt: Stmt.Print): void;
    visitExprInvalid(expr: Expr.Invalid): void;
    visitBinary(expr: Expr.Binary): void;
    visitUnary(expr: Expr.Unary): void;
    visitFactor(expr: Expr.Factor): void;
    visitSuffix(expr: Expr.Suffix): void;
    visitLambda(expr: Expr.Lambda): void;
    visitSuffixTermInvalid(suffixTerm: SuffixTerm.Invalid): void;
    visitSuffixTrailer(suffixTerm: SuffixTerm.SuffixTrailer): void;
    visitSuffixTerm(suffixTerm: SuffixTerm.SuffixTerm): void;
    visitCall(suffixTerm: SuffixTerm.Call): void;
    visitArrayIndex(suffixTerm: SuffixTerm.ArrayIndex): void;
    visitArrayBracket(suffixTerm: SuffixTerm.ArrayBracket): void;
    visitDelegate(suffixTerm: SuffixTerm.Delegate): void;
    visitLiteral(suffixTerm: SuffixTerm.Literal): void;
    visitIdentifier(suffixTerm: SuffixTerm.Identifier): void;
    visitGrouping(suffixTerm: SuffixTerm.Grouping): void;
}
