import { IScanResult } from './types';
/**
 * Class for scanning kerboscript files
 */
export declare class Scanner {
    /**
     * source string for scanning
     */
    private source;
    /**
     * start character of current token
     */
    private start;
    /**
     * current character position
     */
    private current;
    /**
     * current line and character position
     */
    private currentPosition;
    /**
     * start character and line of current token
     */
    private startPosition;
    /**
     * uri of the source file
     */
    private uri;
    /**
     * logger for the scanner
     */
    private readonly logger;
    /**
     * tracer for the scanner
     */
    private readonly tracer;
    /**
     * results for tokens
     */
    private readonly tokenResult;
    /**
     * results for whitespace
     */
    private readonly whiteSpaceResult;
    /**
     * results for diagnostic
     */
    private readonly diagnosticResult;
    /**
     * Scanner constructor
     * @param source source string
     * @param uri source uri
     * @param logger logger for scanner
     * @param tracer tracer for scanner
     */
    constructor(source: string, uri?: string, logger?: ILogger, tracer?: ITracer);
    /**
     * scan all available tokens
     */
    scanTokens(): IScanResult;
    /**
     * Scan a single token
     */
    private scanToken;
    /**
     * extract any identifiers
     */
    private identifier;
    /**
     * extract a file identifier
     */
    private fileIdentifier;
    /**
     * extract a string
     */
    private string;
    /**
     * extract a number
     */
    private number;
    /**
     * advance a number as either an int or double
     */
    private advanceNumber;
    /**
     * advance a number before or after a period
     */
    private advanceNumberComponent;
    /**
     * advance through whitespace
     */
    private advanceWhitespace;
    /**
     * remove underscores from number and generate token
     * @param isFloat is the token a float
     */
    private generateNumber;
    /**
     * generate token from provided token type and optional literal
     * @param type token type
     * @param literal optional literal
     * @param toLower should the lexeme be lowered
     */
    private generateToken;
    /**
     * generate error
     * @param message error message
     */
    private generateError;
    /**
     * increment line
     */
    private incrementLine;
    /**
     * Decrement the file pointer 1 character
     */
    private decrement;
    /**
     * Increment the file pointer 1 character
     */
    private increment;
    /**
     * Is the pointer current at the end of the file
     */
    private isAtEnd;
    /**
     * Peek ahead one character, return null character if past
     * the end
     */
    private peekNext;
    /**
     * Peek the current chacter return null character is past
     * the end
     */
    private peek;
    /**
     * Advance the pointer 1 character returning the current
     * character
     */
    private advance;
    /**
     * Match a character, returns if match was found
     * @param expected character
     */
    private match;
    /**
     * Is the character whitespace
     * @param c character to inspect
     */
    private isWhitespace;
    /**
     * Is the character a digit
     * @param c character to inspect
     */
    private isDigit;
    /**
     * Is the character an alphabet character
     * @param c character to inspect
     */
    private isAlpha;
    /**
     * Is the current character an ascii character
     * @param c character to inspect
     */
    private isAscii;
    /**
     * Is the character alpha numeric
     * @param c character to inspect
     */
    private isAlphaNumeric;
}
