"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const fsUtils_1 = require("../utilities/fsUtils");
const tokentypes_1 = require("../entities/tokentypes");
const arrayUtils_1 = require("../utilities/arrayUtils");
const scanner_1 = require("../scanner/scanner");
const testDir = path_1.join(__dirname, '../../../kerboscripts/parser_valid/');
describe('Scan all files', () => {
    test('scan all', () => {
        fsUtils_1.walkDir(testDir, filePath => {
            const kosFile = fs_1.readFileSync(filePath, 'utf8');
            const scanner = new scanner_1.Scanner(kosFile);
            const { tokens, scanErrors } = scanner.scanTokens();
            const errorResult = scanErrors.map(error => ({ filePath, ...error }));
            expect(tokens.length > 0).toBe(true);
            expect(errorResult.length === 0).toBe(true);
        });
    });
});
const scannerPath = path_1.join(__dirname, '../../../kerboscripts/parser_valid/unitTests/scannertest.ks');
const sequence = [
    tokentypes_1.TokenType.atSign,
    tokentypes_1.TokenType.lazyGlobal,
    tokentypes_1.TokenType.on,
    tokentypes_1.TokenType.period,
    tokentypes_1.TokenType.global,
    tokentypes_1.TokenType.function,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.curlyOpen,
    tokentypes_1.TokenType.parameter,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.comma,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.is,
    tokentypes_1.TokenType.integer,
    tokentypes_1.TokenType.period,
    tokentypes_1.TokenType.print,
    tokentypes_1.TokenType.bracketOpen,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.bracketClose,
    tokentypes_1.TokenType.period,
    tokentypes_1.TokenType.for,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.in,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.curlyOpen,
    tokentypes_1.TokenType.print,
    tokentypes_1.TokenType.bracketOpen,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.bracketClose,
    tokentypes_1.TokenType.period,
    tokentypes_1.TokenType.curlyClose,
    tokentypes_1.TokenType.from,
    tokentypes_1.TokenType.curlyOpen,
    tokentypes_1.TokenType.local,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.is,
    tokentypes_1.TokenType.integer,
    tokentypes_1.TokenType.period,
    tokentypes_1.TokenType.curlyClose,
    tokentypes_1.TokenType.until,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.greater,
    tokentypes_1.TokenType.integer,
    tokentypes_1.TokenType.step,
    tokentypes_1.TokenType.curlyOpen,
    tokentypes_1.TokenType.set,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.to,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.plus,
    tokentypes_1.TokenType.double,
    tokentypes_1.TokenType.period,
    tokentypes_1.TokenType.curlyClose,
    tokentypes_1.TokenType.do,
    tokentypes_1.TokenType.curlyOpen,
    tokentypes_1.TokenType.wait,
    tokentypes_1.TokenType.until,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.period,
    tokentypes_1.TokenType.curlyClose,
    tokentypes_1.TokenType.curlyClose,
    tokentypes_1.TokenType.lock,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.to,
    tokentypes_1.TokenType.string,
    tokentypes_1.TokenType.period,
    tokentypes_1.TokenType.on,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.curlyOpen,
    tokentypes_1.TokenType.stage,
    tokentypes_1.TokenType.period,
    tokentypes_1.TokenType.clearscreen,
    tokentypes_1.TokenType.period,
    tokentypes_1.TokenType.log,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.colon,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.to,
    tokentypes_1.TokenType.string,
    tokentypes_1.TokenType.period,
    tokentypes_1.TokenType.curlyClose,
    tokentypes_1.TokenType.unlock,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.period,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.bracketOpen,
    tokentypes_1.TokenType.list,
    tokentypes_1.TokenType.bracketOpen,
    tokentypes_1.TokenType.bracketClose,
    tokentypes_1.TokenType.comma,
    tokentypes_1.TokenType.integer,
    tokentypes_1.TokenType.bracketClose,
    tokentypes_1.TokenType.period,
    tokentypes_1.TokenType.identifier,
    tokentypes_1.TokenType.off,
    tokentypes_1.TokenType.period,
    tokentypes_1.TokenType.runPath,
    tokentypes_1.TokenType.bracketOpen,
    tokentypes_1.TokenType.string,
    tokentypes_1.TokenType.bracketClose,
    tokentypes_1.TokenType.period,
    tokentypes_1.TokenType.reboot,
    tokentypes_1.TokenType.period,
];
describe('Scan test file', () => {
    test('token sequence', () => {
        const kosFile = fs_1.readFileSync(scannerPath, 'utf8');
        const scanner = new scanner_1.Scanner(kosFile);
        const { tokens, scanErrors } = scanner.scanTokens();
        expect(scanErrors.length === 0).toBe(true);
        for (const [type, token] of arrayUtils_1.zip(sequence, tokens)) {
            // console.log(`${TokenType[token.type]} vs ${TokenType[type]}`);
            expect(token.type).toBe(type);
        }
    });
});
//# sourceMappingURL=scanner.spec.js.map