"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typeUitlities_1 = require("../typeChecker/typeUitlities");
const string_1 = require("../typeChecker/types/primitives/string");
const boolean_1 = require("../typeChecker/types/primitives/boolean");
const structure_1 = require("../typeChecker/types/primitives/structure");
const part_1 = require("../typeChecker/types/parts/part");
const dockingPort_1 = require("../typeChecker/types/parts/dockingPort");
const initialize_1 = require("../typeChecker/types/primitives/initialize");
const initialize_2 = require("../typeChecker/types/orbital/initialize");
const typeCreators_1 = require("../typeChecker/typeCreators");
initialize_1.primitiveInitializer();
initialize_2.oribitalInitializer();
describe('Type Utilities', () => {
    test('Call type', () => {
        expect(typeUitlities_1.isCorrectCallType(2 /* call */, 2 /* call */)).toBe(true);
        expect(typeUitlities_1.isCorrectCallType(2 /* call */, 0 /* get */)).toBe(false);
        expect(typeUitlities_1.isCorrectCallType(2 /* call */, 1 /* set */)).toBe(false);
        expect(typeUitlities_1.isCorrectCallType(2 /* call */, 3 /* optionalCall */)).toBe(false);
        expect(typeUitlities_1.isCorrectCallType(0 /* get */, 2 /* call */)).toBe(false);
        expect(typeUitlities_1.isCorrectCallType(0 /* get */, 0 /* get */)).toBe(true);
        expect(typeUitlities_1.isCorrectCallType(0 /* get */, 1 /* set */)).toBe(false);
        expect(typeUitlities_1.isCorrectCallType(0 /* get */, 3 /* optionalCall */)).toBe(false);
        expect(typeUitlities_1.isCorrectCallType(1 /* set */, 2 /* call */)).toBe(false);
        expect(typeUitlities_1.isCorrectCallType(1 /* set */, 0 /* get */)).toBe(false);
        expect(typeUitlities_1.isCorrectCallType(1 /* set */, 1 /* set */)).toBe(true);
        expect(typeUitlities_1.isCorrectCallType(1 /* set */, 3 /* optionalCall */)).toBe(false);
        expect(typeUitlities_1.isCorrectCallType(3 /* optionalCall */, 2 /* call */)).toBe(true);
        expect(typeUitlities_1.isCorrectCallType(3 /* optionalCall */, 0 /* get */)).toBe(true);
        expect(typeUitlities_1.isCorrectCallType(3 /* optionalCall */, 1 /* set */)).toBe(false);
        expect(typeUitlities_1.isCorrectCallType(3 /* optionalCall */, 3 /* optionalCall */)).toBe(true);
    });
    test('Is Subtype 1', () => {
        expect(typeUitlities_1.isSubType(string_1.stringType, string_1.stringType)).toBe(true);
        expect(typeUitlities_1.isSubType(string_1.stringType, boolean_1.booleanType)).toBe(false);
        expect(typeUitlities_1.isSubType(string_1.stringType, structure_1.structureType)).toBe(true);
        expect(typeUitlities_1.isSubType(string_1.stringType, part_1.partType)).toBe(false);
        expect(typeUitlities_1.isSubType(string_1.stringType, dockingPort_1.dockingPortType)).toBe(false);
        expect(typeUitlities_1.isSubType(boolean_1.booleanType, string_1.stringType)).toBe(false);
        expect(typeUitlities_1.isSubType(boolean_1.booleanType, boolean_1.booleanType)).toBe(true);
        expect(typeUitlities_1.isSubType(boolean_1.booleanType, structure_1.structureType)).toBe(true);
        expect(typeUitlities_1.isSubType(boolean_1.booleanType, part_1.partType)).toBe(false);
        expect(typeUitlities_1.isSubType(boolean_1.booleanType, dockingPort_1.dockingPortType)).toBe(false);
        expect(typeUitlities_1.isSubType(structure_1.structureType, boolean_1.booleanType)).toBe(false);
        expect(typeUitlities_1.isSubType(structure_1.structureType, boolean_1.booleanType)).toBe(false);
        expect(typeUitlities_1.isSubType(structure_1.structureType, structure_1.structureType)).toBe(true);
        expect(typeUitlities_1.isSubType(structure_1.structureType, part_1.partType)).toBe(false);
        expect(typeUitlities_1.isSubType(structure_1.structureType, dockingPort_1.dockingPortType)).toBe(false);
        expect(typeUitlities_1.isSubType(part_1.partType, string_1.stringType)).toBe(false);
        expect(typeUitlities_1.isSubType(part_1.partType, boolean_1.booleanType)).toBe(false);
        expect(typeUitlities_1.isSubType(part_1.partType, structure_1.structureType)).toBe(true);
        expect(typeUitlities_1.isSubType(part_1.partType, part_1.partType)).toBe(true);
        expect(typeUitlities_1.isSubType(part_1.partType, dockingPort_1.dockingPortType)).toBe(false);
        expect(typeUitlities_1.isSubType(dockingPort_1.dockingPortType, string_1.stringType)).toBe(false);
        expect(typeUitlities_1.isSubType(dockingPort_1.dockingPortType, boolean_1.booleanType)).toBe(false);
        expect(typeUitlities_1.isSubType(dockingPort_1.dockingPortType, structure_1.structureType)).toBe(true);
        expect(typeUitlities_1.isSubType(dockingPort_1.dockingPortType, part_1.partType)).toBe(true);
        expect(typeUitlities_1.isSubType(dockingPort_1.dockingPortType, dockingPort_1.dockingPortType)).toBe(true);
    });
    test('Is Subtype 2', () => {
        const aType = typeCreators_1.createStructureType('a');
        const bType = typeCreators_1.createStructureType('b');
        const cType = typeCreators_1.createStructureType('c');
        typeUitlities_1.addPrototype(bType, aType);
        typeUitlities_1.addPrototype(cType, aType);
        expect(typeUitlities_1.isSubType(bType, aType)).toBe(true);
        expect(typeUitlities_1.isSubType(cType, aType)).toBe(true);
        expect(typeUitlities_1.isSubType(aType, bType)).toBe(false);
        expect(typeUitlities_1.isSubType(cType, bType)).toBe(false);
        expect(typeUitlities_1.isSubType(bType, cType)).toBe(false);
        expect(typeUitlities_1.isSubType(aType, cType)).toBe(false);
    });
    test('Has Suffix', () => {
        const aType = typeCreators_1.createStructureType('a');
        const bType = typeCreators_1.createStructureType('b');
        const cType = typeCreators_1.createStructureType('c');
        const dType = typeCreators_1.createStructureType('d');
        typeUitlities_1.addSuffixes(aType, typeCreators_1.createSuffixType('example1', cType), typeCreators_1.createSuffixType('example2', cType));
        typeUitlities_1.addSuffixes(bType, typeCreators_1.createSuffixType('example3', cType));
        typeUitlities_1.addPrototype(bType, aType);
        typeUitlities_1.addSuffixes(cType, typeCreators_1.createArgSuffixType('example', dType, dType));
        expect(typeUitlities_1.hasSuffix(aType, 'example1')).toBe(true);
        expect(typeUitlities_1.hasSuffix(aType, 'example2')).toBe(true);
        expect(typeUitlities_1.hasSuffix(aType, 'example3')).toBe(false);
        expect(typeUitlities_1.hasSuffix(bType, 'example1')).toBe(true);
        expect(typeUitlities_1.hasSuffix(bType, 'example2')).toBe(true);
        expect(typeUitlities_1.hasSuffix(bType, 'example3')).toBe(true);
        expect(typeUitlities_1.hasSuffix(bType, 'other')).toBe(false);
        expect(typeUitlities_1.hasSuffix(cType, 'example')).toBe(true);
        expect(typeUitlities_1.hasSuffix(cType, 'other')).toBe(false);
        expect(typeUitlities_1.hasSuffix(dType, 'any')).toBe(false);
    });
});
//# sourceMappingURL=typeCheckingUtilities.spec.js.map