"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const path_1 = require("path");
const pathResolver_1 = require("../utilities/pathResolver");
const typeGuards_1 = require("../utilities/typeGuards");
const positionUtils_1 = require("../utilities/positionUtils");
const stringUtils_1 = require("../utilities/stringUtils");
const logger_1 = require("../utilities/logger");
describe('path resolver', () => {
    test('path resolver', () => {
        const pathResolver = new pathResolver_1.PathResolver();
        const range = {
            start: {
                line: 0,
                character: 0,
            },
            end: {
                line: 0,
                character: 1,
            },
        };
        const otherFileLocation = {
            range,
            uri: 'file://example/otherFile.ks',
        };
        const otherDirLocation = {
            range,
            uri: 'file://example/up/upFile.ks',
        };
        const relative1 = ['relative', 'path', 'file.ks'].join('/');
        const relative2 = ['..', 'relative', 'path', 'file.ks'].join('/');
        const absolute = ['0:', 'relative', 'path', 'file.ks'].join('/');
        const weird = ['0:relative', 'path', 'file.ks'].join('/');
        expect(pathResolver.resolveUri(otherFileLocation, relative1)).toBeUndefined();
        expect(pathResolver.resolveUri(otherDirLocation, relative2)).toBeUndefined();
        expect(pathResolver.resolveUri(otherFileLocation, absolute)).toBeUndefined();
        expect(pathResolver.resolveUri(otherFileLocation, weird)).toBeUndefined();
        pathResolver.volume0Path = path_1.join('root', 'example');
        pathResolver.volume0Uri = 'file://example';
        const resolvedPath = path_1.join('root', 'example', 'relative', 'path', 'file.ks');
        const resolvedUri = 'file://example/relative/path/file.ks';
        const relativeResolved1 = pathResolver.resolveUri(otherFileLocation, relative1);
        expect(undefined).not.toBe(relativeResolved1);
        if (!typeGuards_1.empty(relativeResolved1)) {
            expect(relativeResolved1.path).toBe(resolvedPath);
            expect(relativeResolved1.uri).toBe(resolvedUri);
            expect(positionUtils_1.rangeEqual(range, relativeResolved1.caller)).toBe(true);
        }
        const relativeResolved2 = pathResolver.resolveUri(otherDirLocation, relative2);
        expect(undefined).not.toBe(relativeResolved2);
        if (!typeGuards_1.empty(relativeResolved2)) {
            expect(relativeResolved2.path).toBe(resolvedPath);
            expect(relativeResolved2.uri).toBe(resolvedUri);
            expect(positionUtils_1.rangeEqual(range, relativeResolved2.caller)).toBe(true);
        }
        const absoluteResolved = pathResolver.resolveUri(otherFileLocation, absolute);
        expect(undefined).not.toBe(absoluteResolved);
        if (!typeGuards_1.empty(absoluteResolved)) {
            expect(absoluteResolved.path).toBe(resolvedPath);
            expect(absoluteResolved.uri).toBe(resolvedUri);
            expect(positionUtils_1.rangeEqual(range, absoluteResolved.caller)).toBe(true);
        }
        const weirdResolved = pathResolver.resolveUri(otherFileLocation, weird);
        expect(undefined).not.toBe(weirdResolved);
        if (!typeGuards_1.empty(weirdResolved)) {
            expect(weirdResolved.path).toBe(resolvedPath);
            expect(weirdResolved.uri).toBe(resolvedUri);
            expect(positionUtils_1.rangeEqual(range, weirdResolved.caller)).toBe(true);
        }
    });
});
const createRange = (startLine, startCharacter, endLine, endCharacter) => {
    return {
        start: vscode_languageserver_1.Position.create(startLine, startCharacter),
        end: vscode_languageserver_1.Position.create(endLine, endCharacter),
    };
};
describe('position utils', () => {
    test('position utils', () => {
        const pos1 = {
            line: 5,
            character: 10,
        };
        const pos2 = {
            line: 5,
            character: 11,
        };
        const pos3 = {
            line: 4,
            character: 2,
        };
        const pos4 = {
            line: 4,
            character: 8,
        };
        // pos 1
        expect(positionUtils_1.positionEqual(pos1, pos1)).toBeTruthy();
        expect(positionUtils_1.positionAfterEqual(pos1, pos1)).toBeTruthy();
        expect(positionUtils_1.positionBeforeEqual(pos1, pos1)).toBeTruthy();
        expect(positionUtils_1.positionBefore(pos1, pos2)).toBeTruthy();
        expect(positionUtils_1.positionBeforeEqual(pos1, pos2)).toBeTruthy();
        expect(positionUtils_1.positionAfter(pos1, pos3)).toBeTruthy();
        expect(positionUtils_1.positionAfterEqual(pos1, pos3)).toBeTruthy();
        expect(positionUtils_1.positionAfter(pos1, pos4)).toBeTruthy();
        expect(positionUtils_1.positionAfterEqual(pos1, pos4)).toBeTruthy();
        // pos 2
        expect(positionUtils_1.positionEqual(pos2, pos2)).toBeTruthy();
        expect(positionUtils_1.positionAfterEqual(pos2, pos2)).toBeTruthy();
        expect(positionUtils_1.positionBeforeEqual(pos2, pos2)).toBeTruthy();
        expect(positionUtils_1.positionAfter(pos2, pos1)).toBeTruthy();
        expect(positionUtils_1.positionAfter(pos2, pos3)).toBeTruthy();
        expect(positionUtils_1.positionAfter(pos2, pos4)).toBeTruthy();
        // pos 3
        expect(positionUtils_1.positionEqual(pos3, pos3)).toBeTruthy();
        expect(positionUtils_1.positionAfterEqual(pos3, pos3)).toBeTruthy();
        expect(positionUtils_1.positionBeforeEqual(pos3, pos3)).toBeTruthy();
        expect(positionUtils_1.positionBefore(pos3, pos1)).toBeTruthy();
        expect(positionUtils_1.positionBefore(pos3, pos2)).toBeTruthy();
        expect(positionUtils_1.positionBefore(pos3, pos4)).toBeTruthy();
        // pos 4
        expect(positionUtils_1.positionEqual(pos4, pos4)).toBeTruthy();
        expect(positionUtils_1.positionAfterEqual(pos4, pos4)).toBeTruthy();
        expect(positionUtils_1.positionBeforeEqual(pos4, pos4)).toBeTruthy();
        expect(positionUtils_1.positionBefore(pos4, pos1)).toBeTruthy();
        expect(positionUtils_1.positionBefore(pos4, pos2)).toBeTruthy();
        expect(positionUtils_1.positionAfter(pos4, pos3)).toBeTruthy();
        expect(positionUtils_1.positionToString(pos1)).toBe('line: 6 character: 11');
    });
    test('ranger utils', () => {
        const range1 = {
            start: {
                line: 4,
                character: 2,
            },
            end: {
                line: 5,
                character: 18,
            },
        };
        const rangeWithin = {
            start: {
                line: 5,
                character: 4,
            },
            end: {
                line: 5,
                character: 16,
            },
        };
        const rangeIntersect = {
            start: {
                line: 4,
                character: 1,
            },
            end: {
                line: 4,
                character: 14,
            },
        };
        const rangeOther = {
            start: {
                line: 4,
                character: 1,
            },
            end: {
                line: 4,
                character: 1,
            },
        };
        // pos 1
        expect(positionUtils_1.rangeEqual(range1, range1)).toBeTruthy();
        expect(positionUtils_1.rangeEqual(rangeWithin, rangeWithin)).toBeTruthy();
        expect(positionUtils_1.rangeEqual(rangeIntersect, rangeIntersect)).toBeTruthy();
        expect(positionUtils_1.rangeContainsPos(range1, rangeWithin.start)).toBeTruthy();
        expect(positionUtils_1.rangeContainsPos(range1, rangeWithin.end)).toBeTruthy();
        expect(positionUtils_1.rangeIntersection(range1, rangeIntersect)).toBeTruthy();
        expect(positionUtils_1.rangeIntersection(rangeIntersect, range1)).toBeTruthy();
        expect(positionUtils_1.rangeAfter(range1, { line: 3, character: 24 })).toBeTruthy();
        expect(positionUtils_1.rangeAfter(range1, { line: 4, character: 1 })).toBeTruthy();
        expect(positionUtils_1.rangeBefore(range1, { line: 5, character: 20 })).toBeTruthy();
        expect(positionUtils_1.rangeBefore(range1, { line: 6, character: 1 })).toBeTruthy();
        expect(positionUtils_1.rangeToString(range1)).toBe('line: 5 character: 3 to line: 6 character: 19');
        expect(positionUtils_1.rangeToString(rangeWithin)).toBe('line: 6 character: 5-17');
        expect(positionUtils_1.rangeToString(rangeIntersect)).toBe('line: 5 character: 2-15');
        expect(positionUtils_1.rangeToString(rangeOther)).toBe('line: 5 character: 2');
    });
    test('binary search utils', () => {
        const ranges = [
            createRange(0, 0, 0, 5),
            createRange(0, 6, 0, 10),
            createRange(0, 11, 0, 15),
            createRange(0, 21, 0, 25),
            createRange(0, 26, 0, 30),
            createRange(0, 31, 0, 35),
        ];
        const unity = (x) => x;
        const result11 = positionUtils_1.binaryLeft(ranges, vscode_languageserver_1.Position.create(0, 1));
        const result12 = positionUtils_1.binaryLeftKey(ranges, vscode_languageserver_1.Position.create(0, 1), unity);
        expect(result11).toBe(ranges[0]);
        expect(result12).toBe(ranges[0]);
        const result21 = positionUtils_1.binaryLeft(ranges, vscode_languageserver_1.Position.create(0, 17));
        const result22 = positionUtils_1.binaryLeftKey(ranges, vscode_languageserver_1.Position.create(0, 17), unity);
        expect(result21).toBe(ranges[2]);
        expect(result22).toBe(ranges[2]);
        const result31 = positionUtils_1.binaryLeft(ranges, vscode_languageserver_1.Position.create(0, 26));
        const result32 = positionUtils_1.binaryLeftKey(ranges, vscode_languageserver_1.Position.create(0, 26), unity);
        expect(result31).toBe(ranges[4]);
        expect(result32).toBe(ranges[4]);
        const result41 = positionUtils_1.binaryRight(ranges, vscode_languageserver_1.Position.create(0, 1));
        const result42 = positionUtils_1.binaryRightKey(ranges, vscode_languageserver_1.Position.create(0, 1), unity);
        expect(result41).toBe(ranges[0]);
        expect(result42).toBe(ranges[0]);
        const result51 = positionUtils_1.binaryRight(ranges, vscode_languageserver_1.Position.create(0, 17));
        const result52 = positionUtils_1.binaryRightKey(ranges, vscode_languageserver_1.Position.create(0, 17), unity);
        expect(result51).toBe(ranges[3]);
        expect(result52).toBe(ranges[3]);
        const result61 = positionUtils_1.binaryRight(ranges, vscode_languageserver_1.Position.create(0, 26));
        const result62 = positionUtils_1.binaryRightKey(ranges, vscode_languageserver_1.Position.create(0, 26), unity);
        expect(result61).toBe(ranges[4]);
        expect(result62).toBe(ranges[4]);
    });
});
describe('to case', () => {
    test('case changes', () => {
        expect(stringUtils_1.toCase(0 /* lowercase */, 'example')).toBe('example');
        expect(stringUtils_1.toCase(1 /* uppercase */, 'example')).toBe('EXAMPLE');
        expect(stringUtils_1.toCase(3 /* pascalcase */, 'example')).toBe('Example');
        expect(stringUtils_1.toCase(2 /* camelcase */, 'example')).toBe('example');
        expect(stringUtils_1.toCase(0 /* lowercase */, 'EXAMPLE')).toBe('example');
        expect(stringUtils_1.toCase(1 /* uppercase */, 'EXAMPLE')).toBe('EXAMPLE');
        expect(stringUtils_1.toCase(3 /* pascalcase */, 'EXAMPLE')).toBe('Example');
        expect(stringUtils_1.toCase(2 /* camelcase */, 'EXAMPLE')).toBe('example');
        expect(stringUtils_1.toCase(0 /* lowercase */, 'EXAMPLE', 'example')).toBe('exampleexample');
        expect(stringUtils_1.toCase(1 /* uppercase */, 'EXAMPLE', 'example')).toBe('EXAMPLEEXAMPLE');
        expect(stringUtils_1.toCase(3 /* pascalcase */, 'EXAMPLE', 'example')).toBe('ExampleExample');
        expect(stringUtils_1.toCase(2 /* camelcase */, 'EXAMPLE', 'example')).toBe('exampleExample');
    });
});
describe('logger', () => {
    test('log level', () => {
        const mockBase = {
            lastLevel: 0 /* verbose */,
            lastMessage: '',
            log(message) {
                mockBase.lastLevel = 2 /* log */;
                mockBase.lastMessage = message;
            },
            info(message) {
                mockBase.lastLevel = 1 /* info */;
                mockBase.lastMessage = message;
            },
            warn(message) {
                mockBase.lastLevel = 3 /* warn */;
                mockBase.lastMessage = message;
            },
            error(message) {
                mockBase.lastLevel = 4 /* error */;
                mockBase.lastMessage = message;
            },
        };
        const logger = new logger_1.Logger(mockBase, 1 /* info */);
        expect(logger.level).toBe(1 /* info */);
        logger.info('info');
        expect(mockBase.lastLevel).toBe(1 /* info */);
        expect(mockBase.lastMessage).toBe('info');
        logger.log('log');
        expect(mockBase.lastLevel).toBe(2 /* log */);
        expect(mockBase.lastMessage).toBe('log');
        logger.warn('warn');
        expect(mockBase.lastLevel).toBe(3 /* warn */);
        expect(mockBase.lastMessage).toBe('warn');
        // TODO this is a temporary thing until we get the
        // errors in the type checker under control
        logger.error('error');
        expect(mockBase.lastLevel).toBe(3 /* warn */);
        expect(mockBase.lastMessage).toBe('error');
        // set logging off
        logger.level = 5 /* none */;
        logger.info('info');
        expect(mockBase.lastLevel).toBe(3 /* warn */);
        expect(mockBase.lastMessage).toBe('error');
        logger.log('log');
        expect(mockBase.lastLevel).toBe(3 /* warn */);
        expect(mockBase.lastMessage).toBe('error');
        logger.warn('warn');
        expect(mockBase.lastLevel).toBe(3 /* warn */);
        expect(mockBase.lastMessage).toBe('error');
        logger.error('error');
        expect(mockBase.lastLevel).toBe(3 /* warn */);
        expect(mockBase.lastMessage).toBe('error');
    });
});
// describe('tree traverse', () => {
//   test('token check', () => {
//     const source = readFileSync('../../kerboscripts/unitTests/allLanguage.ks', 'utf-8');
//     const scanner = new Scanner(source, 'file://fake.ks');
//     const { tokens } = scanner.scanTokens();
//     const parser = new Parser('file:://fake.ks', tokens);
//     const { script } = parser.parse();
//     const tokenCheck = new TokenCheck();
//     tokenCheck.orderedTokens(script);
//     for (const statement of validStatements) {
//     }
//   });
// });
//# sourceMappingURL=utilitites.spec.js.map