import { IGenericSuffixType, IGenericBasicType, IBasicType, ISuffixType, IVariadicType, IFunctionType } from './types/types';
/**
 * Generate a new basic generic type
 * @param name name of the new generic type
 */
export declare const createGenericBasicType: (name: string) => IGenericBasicType;
/**
 * Generate a new type parameter
 * @param name naem of the type parameter
 */
export declare const getTypeParameter: (arg1: string) => IGenericBasicType;
/**
 * The basic type parameter t
 */
export declare const tType: IGenericBasicType;
/**
 * Generate a new basic type
 * @param name name of the new type
 */
export declare const createStructureType: (name: string) => IBasicType;
/**
 * Generate a new generic callable suffix type
 * @param name name of the suffix
 * @param returns return type of suffix call
 * @param params parameters of suffix call
 */
export declare const createGenericArgSuffixType: (name: string, returns: IGenericBasicType, ...params: IGenericBasicType[]) => IGenericSuffixType;
/**
 * Generate a new callable suffix type
 * @param name name of the suffix
 * @param returns return type of suffix call
 * @param params parameters of suffix call
 */
export declare const createArgSuffixType: (name: string, returns: IBasicType, ...params: IBasicType[]) => ISuffixType;
/**
 * Generate a new get only suffix type
 * @param name name of the suffix
 * @param returns return type of suffix
 */
export declare const createSuffixType: (name: string, returns: IBasicType) => ISuffixType;
/**
 * Generate a new get and set suffix type
 * @param name name of the suffix
 * @param returns return type of suffix
 */
export declare const createSetSuffixType: (name: string, returns: IBasicType) => ISuffixType;
/**
 * Generate a new callable variadic suffix type
 * @param name name of the suffix type
 * @param returns return type of suffix call
 * @param params variadic type of parameters
 */
export declare const createVarSuffixType: (name: string, returns: IBasicType, params: IVariadicType) => ISuffixType;
/**
 * Create variadic type
 */
export declare const createVarType: (arg1: IBasicType) => IVariadicType;
/**
 * Generate a new function type
 * @param name name of the function
 * @param returns return type of function call
 * @param params parameters of function
 */
export declare const createFunctionType: (name: string, returns: IBasicType, ...params: IBasicType[]) => IFunctionType;
/**
 * Generate a new variadic function type
 * @param name name of the function
 * @param returns return type of function call
 * @param params variadic type of function call parameters
 */
export declare const createVarFunctionType: (name: string, returns: IBasicType, params: IVariadicType) => IFunctionType;
