"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const memoize_1 = require("../utilities/memoize");
const ksType_1 = require("./ksType");
/**
 * Generate a new basic generic type
 * @param name name of the new generic type
 */
exports.createGenericBasicType = (name) => {
    return new ksType_1.GenericBasicType(name);
};
/**
 * Generate a new type parameter
 * @param name naem of the type parameter
 */
exports.getTypeParameter = memoize_1.memoize((name) => {
    return exports.createGenericBasicType(name);
});
/**
 * The basic type parameter t
 */
exports.tType = exports.getTypeParameter('T');
/**
 * Generate a new basic type
 * @param name name of the new type
 */
exports.createStructureType = (name) => {
    return new ksType_1.BasicType(name);
};
/**
 * Generate a new generic callable suffix type
 * @param name name of the suffix
 * @param returns return type of suffix call
 * @param params parameters of suffix call
 */
exports.createGenericArgSuffixType = (name, returns, ...params) => {
    const callType = params.length > 0 ? 2 /* call */ : 3 /* optionalCall */;
    return new ksType_1.GenericSuffixType(name.toLowerCase(), callType, params, returns);
};
/**
 * Generate a new callable suffix type
 * @param name name of the suffix
 * @param returns return type of suffix call
 * @param params parameters of suffix call
 */
exports.createArgSuffixType = (name, returns, ...params) => {
    const callType = params.length > 0 ? 2 /* call */ : 3 /* optionalCall */;
    return new ksType_1.SuffixType(name.toLowerCase(), callType, params, returns);
};
/**
 * Generate a new get only suffix type
 * @param name name of the suffix
 * @param returns return type of suffix
 */
exports.createSuffixType = (name, returns) => {
    return new ksType_1.SuffixType(name.toLowerCase(), 0 /* get */, [], returns);
};
/**
 * Generate a new get and set suffix type
 * @param name name of the suffix
 * @param returns return type of suffix
 */
exports.createSetSuffixType = (name, returns) => {
    return new ksType_1.SuffixType(name.toLowerCase(), 1 /* set */, [], returns);
};
/**
 * Generate a new callable variadic suffix type
 * @param name name of the suffix type
 * @param returns return type of suffix call
 * @param params variadic type of parameters
 */
exports.createVarSuffixType = (name, returns, params) => {
    return new ksType_1.SuffixType(name.toLowerCase(), 3 /* optionalCall */, params, returns);
};
/**
 * Create variadic type
 */
exports.createVarType = memoize_1.memoize((type) => {
    return new ksType_1.VariadicType(type);
});
/**
 * Generate a new function type
 * @param name name of the function
 * @param returns return type of function call
 * @param params parameters of function
 */
exports.createFunctionType = (name, returns, ...params) => {
    const callType = params.length > 0 ? 2 /* call */ : 3 /* optionalCall */;
    return new ksType_1.FunctionType(name.toLowerCase(), callType, params, returns);
};
/**
 * Generate a new variadic function type
 * @param name name of the function
 * @param returns return type of function call
 * @param params variadic type of function call parameters
 */
exports.createVarFunctionType = (name, returns, params) => {
    return new ksType_1.FunctionType(name.toLowerCase(), 3 /* optionalCall */, params, returns);
};
//# sourceMappingURL=typeCreators.js.map