import { IGenericVariadicType, IGenericSuffixType, ISuffixType, IVariadicType, IGenericBasicType, Operator, Type, IBasicType, CallType } from './types/types';
import { Token } from '../entities/token';
/**
 * Retrieve the type of the follow token
 * @param token token to retreive
 */
export declare const tokenTrackedType: (token: Token) => Maybe<Type>;
/**
 * check if the target call type is compatable with real call type
 * @param queryCallType real call type
 * @param targetCallType query call type
 */
export declare const isCorrectCallType: (queryCallType: CallType, targetCallType: CallType) => boolean;
/**
 * check to see type is a sub type of target type
 * @param queryType query type
 * @param targetType target type
 */
export declare const isSubType: (queryType: Type, targetType: Type) => boolean;
/**
 * Does the given type have the requested operator
 * @param type type
 * @param operator operator
 */
export declare const hasOperator: (type: Type, operator: Operator) => IBasicType | undefined;
/**
 * Does the given type have the requested suffix
 * @param type type
 * @param suffix suffix string
 */
export declare const hasSuffix: (type: Type, suffix: string) => boolean;
/**
 * Get the provided suffix from the type if it exists
 * @param type type
 * @param suffix suffix string
 */
export declare const getSuffix: (type: Type, suffix: string) => Maybe<ISuffixType>;
/**
 * Retreive all suffixes from the given type
 * @param type type
 */
export declare const allSuffixes: (type: Type) => ISuffixType[];
/**
 * Is the generic variadic type a full variadic type
 * @param type maybe full variadic type
 */
export declare const isFullVarType: (type: IGenericVariadicType) => type is IVariadicType;
/**
 * Is the generic arguemnt type a full argument type
 * @param type maybe full argument type
 */
export declare const isFullType: (type: IGenericBasicType) => type is IBasicType;
/**
 * Add type to prototype chain
 * @param type type to add prototype
 * @param prototype prototype
 */
export declare const addPrototype: <T extends IGenericBasicType>(type: T, prototype: T) => void;
/**
 * Add operator to type
 * @param type type to add operator
 * @param operators operators
 */
export declare const addOperators: <T extends IGenericBasicType>(type: T, ...operators: [Operator, IBasicType][]) => void;
/**
 * Add suffixes to type
 * @param type type to add suffixes
 * @param suffixes suffixes
 */
export declare const addSuffixes: <T extends IGenericBasicType, S extends IGenericSuffixType>(type: T, ...suffixes: S[]) => void;
