import { SuffixTracker } from '../../analysis/suffixTracker';
interface ITypeMeta<T> {
    toTypeString(): string;
    toConcreteType(type: ArgumentType): T;
}
export declare const enum TypeKind {
    basic = 0,
    variadic = 1,
    suffix = 2,
    function = 3
}
export declare const enum CallType {
    get = 0,
    set = 1,
    call = 2,
    optionalCall = 3
}
export declare const enum Operator {
    plus = 0,
    subtract = 1,
    multiply = 2,
    divide = 3,
    power = 4,
    greaterThan = 5,
    lessThan = 6,
    greaterThanEqual = 7,
    lessThanEqual = 8,
    notEqual = 9,
    equal = 10
}
export interface ITemplateBasicType<TSuffixType, TConcreteType> extends ITypeMeta<TConcreteType> {
    name: string;
    suffixes: Map<string, TSuffixType>;
    operators: Map<Operator, TConcreteType>;
    inherentsFrom?: ITemplateBasicType<TSuffixType, TConcreteType>;
    fullType: boolean;
}
export interface ITemplateSuffixType<TBasicType, TVariadicType, TConcreteType> extends ITypeMeta<TConcreteType> {
    name: string;
    callType: CallType;
    params: TBasicType[] | TVariadicType;
    returns: TBasicType;
    fullType: boolean;
}
export interface IGenericBasicType extends ITemplateBasicType<IGenericSuffixType, ArgumentType> {
    kind: TypeKind.basic;
}
export interface IGenericSuffixType extends ITemplateSuffixType<IGenericArgumentType, IGenericVariadicType, ISuffixType> {
    kind: TypeKind.suffix;
}
export interface IGenericVariadicType extends ITypeMeta<IVariadicType> {
    type: IGenericBasicType;
    kind: TypeKind.variadic;
}
export declare type IGenericArgumentType = IGenericBasicType;
export interface IBasicType extends IGenericBasicType {
    inherentsFrom?: ArgumentType;
    suffixes: Map<string, ISuffixType>;
    fullType: true;
    kind: TypeKind.basic;
}
export interface ISuffixType extends IGenericSuffixType {
    params: ArgumentType[] | IVariadicType;
    returns: ArgumentType;
    fullType: true;
    kind: TypeKind.suffix;
    getTracker(): SuffixTracker;
}
export interface IVariadicType extends IGenericVariadicType {
    type: IBasicType;
    fullType: true;
    kind: TypeKind.variadic;
}
export declare type ArgumentType = IBasicType;
export interface IConstantType<T> extends IBasicType {
    value: T;
}
export interface IFunctionType extends ITypeMeta<IFunctionType> {
    name: string;
    callType: CallType.call | CallType.optionalCall;
    params: ArgumentType[] | IVariadicType;
    returns: ArgumentType;
    fullType: true;
    kind: TypeKind.function;
}
export declare type Type = ArgumentType | IFunctionType | ISuffixType;
export {};
