"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cleanCompletion = (completion) => {
    return {
        label: completion.label,
        kind: completion.kind,
        detail: completion.detail,
        documentation: typeof completion.documentation === 'object'
            ? exports.cleanMarkup(completion.documentation)
            : completion.documentation,
        deprecated: completion.deprecated,
        preselect: completion.preselect,
        sortText: completion.sortText,
        filterText: completion.filterText,
        insertText: completion.insertText,
        insertTextFormat: completion.insertTextFormat,
        textEdit: completion.textEdit,
        additionalTextEdits: completion.additionalTextEdits &&
            completion.additionalTextEdits.map(edit => exports.cleanTextEdit(edit)),
        commitCharacters: completion.commitCharacters,
        command: completion.command && exports.cleanCommand(completion.command),
        data: completion.data,
    };
};
exports.cleanToken = (token) => {
    return {
        tag: 'token',
        type: token.type,
        lexeme: token.lexeme,
        literal: token.literal,
        start: exports.cleanPosition(token.start),
        end: exports.cleanPosition(token.end),
        uri: token.uri,
        typeString: token.typeString,
        range: exports.cleanRange(token.range),
    };
};
exports.cleanHover = (hover) => {
    return {
        contents: hover.contents,
        range: hover.range && exports.cleanRange(hover.range),
    };
};
exports.cleanMarkup = (markup) => {
    return {
        kind: markup.kind,
        value: markup.value,
    };
};
exports.cleanCommand = (command) => {
    return {
        title: command.title,
        command: command.command,
        arguments: command.arguments && [...command.arguments],
    };
};
exports.cleanTextEdit = (textEdit) => {
    return {
        range: exports.cleanRange(textEdit.range),
        newText: textEdit.newText,
    };
};
exports.cleanDiagnostic = (diagnostics) => {
    return {
        range: exports.cleanRange(diagnostics.range),
        severity: diagnostics.severity,
        code: diagnostics.code,
        source: diagnostics.source,
        message: diagnostics.message,
        relatedInformation: diagnostics.relatedInformation &&
            diagnostics.relatedInformation.map(info => exports.cleanRelatedInformation(info)),
    };
};
exports.cleanRange = (range) => {
    return {
        start: exports.cleanPosition(range.start),
        end: exports.cleanPosition(range.end),
    };
};
exports.cleanPosition = (position) => {
    return {
        line: position.line,
        character: position.character,
    };
};
exports.cleanLocation = (location) => {
    return {
        uri: location.uri,
        range: exports.cleanRange(location.range),
    };
};
exports.cleanRelatedInformation = (relatedInformation) => {
    return {
        location: {
            uri: relatedInformation.location.uri,
            range: exports.cleanRange(relatedInformation.location.range),
        },
        message: relatedInformation.message,
    };
};
//# sourceMappingURL=clean.js.map