"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const stringUtils_1 = require("./stringUtils");
exports.builtIn = 'builtIn';
exports.iterator = 'iterator';
exports.serverName = 'kos-vscode';
exports.languageServer = 'kos-language-server';
exports.keywords = new Set([
    'add',
    'and',
    'all',
    'at',
    'break',
    'clearscreen',
    'compile',
    'copy',
    'do',
    'declare',
    'defined',
    'delete',
    'edit',
    'else',
    'false',
    'file',
    'for',
    'from',
    'function',
    'global',
    'if',
    'in',
    'is',
    'lazyglobal',
    'list',
    'local',
    'lock',
    'log',
    'not',
    'off',
    'on',
    'or',
    'once',
    'parameter',
    'preserve',
    'print',
    'reboot',
    'remove',
    'rename',
    'return',
    'run',
    'runpath',
    'runoncepath',
    'set',
    'shutdown',
    'stage',
    'step',
    'switch',
    'then',
    'to',
    'true',
    'toggle',
    'unlock',
    'unset',
    'until',
    'volume',
    'wait',
    'when',
]);
const keywordSegments = [
    ['add'],
    ['and'],
    ['all'],
    ['at'],
    ['break'],
    ['clear', 'screen'],
    ['compile'],
    ['copy'],
    ['do'],
    ['declare'],
    ['defined'],
    ['delete'],
    ['edit'],
    ['else'],
    ['false'],
    ['file'],
    ['for'],
    ['from'],
    ['function'],
    ['global'],
    ['if'],
    ['in'],
    ['is'],
    ['lazy', 'global'],
    ['list'],
    ['local'],
    ['lock'],
    ['log'],
    ['not'],
    ['off'],
    ['on'],
    ['or'],
    ['once'],
    ['parameter'],
    ['preserve'],
    ['print'],
    ['reboot'],
    ['remove'],
    ['rename'],
    ['return'],
    ['run'],
    ['run', 'path'],
    ['run', 'once', 'path'],
    ['set'],
    ['shutdown'],
    ['stage'],
    ['step'],
    ['switch'],
    ['then'],
    ['to'],
    ['true'],
    ['toggle'],
    ['unlock'],
    ['unset'],
    ['until'],
    ['volume'],
    ['wait'],
    ['when'],
];
/**
 * Generate a cased set of keyword completion items
 * @param caseKind case of the keyword completions
 */
exports.keywordCompletions = (caseKind) => {
    const casedKeywords = [];
    for (const segments of keywordSegments) {
        casedKeywords.push(stringUtils_1.toCase(caseKind, ...segments));
    }
    return casedKeywords.map(keyword => ({
        kind: vscode_languageserver_1.CompletionItemKind.Keyword,
        label: keyword,
        data: undefined,
    }));
};
//# sourceMappingURL=constants.js.map