import * as Stmt from '../parser/stmt';
import { RunStmtType } from '../parser/types';
import { ILoadData } from '../types';
import { Location } from 'vscode-languageserver';
/**
 * Class to resolve run statements or calls to file paths
 */
export declare class PathResolver {
    volume0Path?: string | undefined;
    volume0Uri?: string | undefined;
    private replacer;
    constructor(volume0Path?: string | undefined, volume0Uri?: string | undefined);
    /**
     * Is the resolve ready to resolve paths
     */
    readonly ready: boolean;
    /**
     * Resolve uri to load data
     * @param caller location of caller
     * @param path provided path in run statement or class
     */
    resolveUri(caller: Location, path?: string): Maybe<ILoadData>;
    /**
     * Creates a load data payload from a caller and path segments
     * @param caller call location
     * @param pathSegments path segments
     */
    private loadData;
    /**
     * Convert a path to a uri
     * @param pathSegments path segments to convert
     */
    private pathToUri;
    /**
     * Get the relative path to volume 0: for a uri
     * @param uri requested uri
     */
    relativePath(uri: string): string;
}
/**
 * Get io path, currently only supports string literals
 * @param stmt io statements
 */
export declare const ioPath: (stmt: Stmt.Log | Stmt.Copy | Stmt.Rename | Stmt.Delete) => string | undefined;
/**
 * based on run type determine how to get file path
 * @param stmt run statement
 */
export declare const runPath: (stmt: RunStmtType) => string | undefined;
