import { Position, Range, Location } from 'vscode-languageserver';
/**
 * Is this position after the next
 * @param pos1 the target position
 * @param pos2 the other position
 */
export declare const positionAfter: (pos1: Position, pos2: Position) => boolean;
/**
 * Is this position equal to or after the next
 * @param pos1 the target position
 * @param pos2 the other position
 */
export declare const positionAfterEqual: (pos1: Position, pos2: Position) => boolean;
/**
 * Is this position before the next
 * @param pos1 the target position
 * @param pos2 the other position
 */
export declare const positionBefore: (pos1: Position, pos2: Position) => boolean;
/**
 * Is this position equal or before the next
 * @param pos1 the target position
 * @param pos2 the other position
 */
export declare const positionBeforeEqual: (pos1: Position, pos2: Position) => boolean;
/**
 * Is this position equal to the next
 * @param pos1 the target position
 * @param pos2 the other position
 */
export declare const positionEqual: (pos1: Position, pos2: Position) => boolean;
/**
 * Is this range equal to the next
 * @param range1 the target range
 * @param range2 the other range
 */
export declare const rangeEqual: (range1: Range, range2: Range) => boolean;
/**
 * Does this range contain another prange
 * @param range1 the target range
 * @param range2 the query range
 */
export declare const rangeContains: (range1: Range, range2: Range) => boolean;
/**
 * Does this range contain this position
 * @param range the target range
 * @param pos the query position
 */
export declare const rangeContainsPos: (range: Range, pos: Position) => boolean;
/**
 * Is this range before this position
 * @param range the target range
 * @param pos the query position
 */
export declare const rangeBefore: (range: Range, pos: Position) => boolean;
/**
 * Is this range after this position
 * @param range the target range
 * @param pos the query position
 */
export declare const rangeAfter: (range: Range, pos: Position) => boolean;
/**
 * This is used to sort a set of ranges in to order
 * @param range1 first range
 * @param range2 second range
 */
export declare const rangeOrder: (range1: Range, range2: Range) => number;
/**
 * Does this range intersect the next
 * @param range1 the target range
 * @param range2 the other range
 */
export declare const rangeIntersection: (range1: Range, range2: Range) => boolean;
/**
 * Generate a human readable string of the range
 * @param range range to string
 */
export declare const rangeToString: (range: Range) => string;
/**
 * Generate a human readable string of this position
 * @param pos position to string
 */
export declare const positionToString: (pos: Position) => string;
/**
 * Binary search a collection of ordered ranges
 * @param ranges sorted ranges to search
 * @param pos query position
 */
export declare const binarySearch: <T extends Range>(ranges: T[], pos: Position) => Maybe<T>;
/**
 * Binary search right on a collection of ordered ranges
 * @param ranges sorted ranges to search
 * @param pos query position
 */
export declare const binaryRight: <T extends Range>(ranges: T[], pos: Position) => Maybe<T>;
/**
 * Binary search left on a collection of ordered ranges
 * @param ranges sorted ranges to search
 * @param pos query position
 */
export declare const binaryLeft: <T extends Range>(ranges: T[], pos: Position) => Maybe<T>;
/**
 * Binary search with provided function to generate a range
 * @param ranges sorted collection of items that can generate a range
 * @param pos query position
 * @param key key function to generate a range
 */
export declare const binaryRightKey: <T>(ranges: T[], pos: Position, key: (range: T) => Range) => Maybe<T>;
/**
 * Binary search left with provided function to generate a range
 * @param ranges sorted collection of items that can generate a range
 * @param pos query position
 * @param key key function to generate a range
 */
export declare const binaryLeftKey: <T>(ranges: T[], pos: Position, key: (range: T) => Range) => Maybe<T>;
/**
 * Binary search for index with provided function to generate a range.
 * Returns array if between ranges
 * @param ranges ordered collection of ranges
 * @param pos query position
 */
export declare const binarySearchIndex: <T extends Range>(ranges: T[], pos: Position) => number | [number, number];
/**
 * Binary search for index of matching item using . Returns array if between ranges
 * @param ranges ordered collection of ranges
 * @param pos query position
 */
export declare const binarySearchKeyIndex: <T>(ranges: T[], pos: Position, key: (range: T) => Range) => number | [number, number];
/**
 * Are these locations equal
 * @param loc1 target location
 * @param loc2 other location
 */
export declare const locationEqual: (loc1: Location, loc2: Location) => boolean;
/**
 * Does location 1 contain location 2
 * @param loc1 target location
 * @param loc2 query location
 */
export declare const locationContains: (loc1: Location, loc2: Location) => boolean;
