"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Convert a set of string segments to the requested case
 * @param caseKind requested case
 * @param segments string segments
 */
exports.toCase = (caseKind, ...segments) => {
    switch (caseKind) {
        case 0 /* lowercase */:
            return segments.map(s => s.toLowerCase()).join('');
        case 1 /* uppercase */:
            return segments.map(s => s.toUpperCase()).join('');
        case 2 /* camelcase */:
            return toCamelCase(...segments);
        case 3 /* pascalcase */:
            return toPascalCase(...segments);
    }
};
/**
 * Convert a set of string segments to pascal case
 * @param segments string segments
 */
const toPascalCase = (...segments) => {
    return segments
        .map(s => s.charAt(0).toUpperCase() + s.slice(1).toLowerCase())
        .join('');
};
/**
 * Convert a set of string segemtns to camel case
 * @param segments string segments
 */
const toCamelCase = (...segments) => {
    if (segments.length === 1) {
        return segments[0].toLowerCase();
    }
    const [first, ...rest] = segments;
    return [
        first.toLowerCase(),
        ...rest.map(s => s.charAt(0).toUpperCase() + s.slice(1).toLowerCase()),
    ].join('');
};
//# sourceMappingURL=stringUtils.js.map