/**
 * Maybe type for things that might be undefined
 */
declare type Maybe<T> = T | undefined;
/**
 * Helper type the dual of pick
 */
declare type Omit<T, K extends keyof T> = Pick<T, Exclude<keyof T, K>>;
/**
 * Require certain fields to not be undefined or null
 */
declare type PartialRequire<T, K extends keyof T> = Required<Pick<T, K>> & Omit<T, K>;
/**
 * The type helper restricts the type to constructor functions
 * or classes
 */
declare type Constructor<T = {}> = new (...args: any[]) => T;
/**
 * This type helper gets all property names of type T that extend type U
 */
declare type PropertyNames<T, U> = {
    [K in keyof T]: T[K] extends U ? K : never;
}[keyof T];
/**
 * This type helper get all properties of type T that extend type U
 */
declare type Properties<T, U> = Pick<T, PropertyNames<T, U>>;
/**
 * Create a writable version of a readonly mapped type
 */
declare type Writeable<T> = {
    -readonly [P in keyof T]-?: T[P];
};
/**
 * Currenlty used in signature utilities
 */
interface IdentifierIndex {
    identifier: string;
    index: number;
}
/**
 * Base logger interface, mirror vscodes logger
 */
interface ILoggerBase {
    error(message: string): void;
    warn(message: string): void;
    info(message: string): void;
    log(message: string): void;
}
/**
 * Interface for internal logger
 */
interface ILogger extends ILoggerBase {
    level: LogLevel;
    verbose(message: string): void;
}
/**
 * Logging level internal to kos-language-server
 */
declare const enum LogLevel {
    verbose = 0,
    info = 1,
    log = 2,
    warn = 3,
    error = 4,
    none = 5
}
/**
 * Interface for internal tracer
 */
interface ITracer {
    log(message: string, verbose?: string): void;
}
/**
 * This is used by the various completion functionailities
 * to decide how they should be cased
 */
declare const enum CaseKind {
    lowercase = 0,
    uppercase = 1,
    camelcase = 2,
    pascalcase = 3
}
